/*
   This file is part of the MoCA video library.
   Name: MpegStatusTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/

// MoCA includes
#include <VideoIndex.h>
#include <CreateMocaPath.h>
#include <FileList.h>
#include <VideoDecoder.h>



void usage()
{
	cerr << "Usage: MpegIndex [options] filename|path" << endl;
	cerr << VIDEO_INDEX_VERSION << endl;
	cerr << "get information about MPEG/AVI file" << endl;
	cerr << endl;
	cerr << "options:" << endl;
	cerr << " -recurse      analyses all *.mpg files in path" << endl;
	cerr << endl;
	cerr << "Example:" <<  endl;
	cerr << "MpegIndex test.mpg" << endl;
	cerr << "MpegIndex -recurse /opt/movies/" << endl;
	exit(1);
} // usage

void analyseFile (string &name)
{
	cout << endl  << "processing file: "<< name << endl;
	VideoIndexData data;
	VideoIndex reader (name);
	reader.process(data);
	cout << data << endl;

	VideoDecoder readerVideo(name);
	data=readerVideo.getIndex();
	Image<uint8> img;
	if (data.videoSize[DIM_Z] < 45) {
		for (int32 i=0; i<data.videoSize[DIM_Z]; i++)
			img=readerVideo.getImage(i);
	} else {
		for (int32 i=0; i<20; i++)
			img=readerVideo.getImage(i);
		for (int32 i=data.videoSize[DIM_Z]-20; i<data.videoSize[DIM_Z]; i++)
			img=readerVideo.getImage(i);
	} // else

	return;
}

int main(int argc, char* argv[])
{
	bool recurse=false;
	string filename="";
	string s = "";

	if (argc<2) usage();
	filename=argv[argc-1];
	for (int i=1; i<argc; i++) {
		s=argv[i];
		for (uint32 k=0; k<s.size(); k++)
		  s[k]=toupper(s[k]);

		if (s=="-RECURSE") recurse=true;
		else {
			if (i != argc-1) usage();
		} // else
	} // for


	if (!recurse) {
		analyseFile (filename);
	} else {

		vector<FileInfo> mpgFiles;
		FileList fl(filename, true);    
		fl.process("*.mpg", mpgFiles);

		// open mpeg files
	    vector<FileInfo>::iterator it;
		for (it = mpgFiles.begin(); it != mpgFiles.end(); it++) {
			analyseFile ((*it).name);
		} // for
		cout << endl << endl << mpgFiles.size() << " files found." << endl;
	} // else

	return 0;
} // main




