/*
   This file is part of the MoCA video library.
   Name: SceneDetectionTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/

// open A/V-MPEG file, 
// detect scenes 
// store scene information in data file

// C includes
#include <time.h>

// C++ includes
#include <string>
#include <iostream>
using namespace std;

// MoCA includes
#include <SceneDetection.h>
#include <HistogramFeature.h>


void usage()
{
	cerr << "Usage: SceneDetection [options] filename|path" << endl;
	cerr << endl;
	cerr << "options:" << endl;
	cerr << " -recurse        analyses all *.mpg files in path" << endl;
	cerr << " -select         select relevant shots and scenes" << endl;
	cerr << " -cluster <int>  number of clusters (default: 20 clusters)" << endl;
	cerr << endl;
	cerr << "Example:" <<  endl;
	cerr << "SceneDetection -recurse /opt/movies/" << endl;
	cerr << "SceneDetection -recurse -select \"C:\\test videos\\\"" << endl;
	exit(1);
} // usage


#include<IImageStream.h>
#include<MocaSignal.h>
int main(int argc, char* argv[])
{
	
    long int startTime = time(NULL);
	string filename = "";
	
	
    if (argc<2) usage();
	filename=argv[argc-1];
	string s;
	bool recurse=false;
	bool select=false;
	int numberOfCluster = 20;

	for (int i=1; i<argc; i++) {
		s=argv[i];
		for (uint32 k=0; k<s.size(); k++)
		  s[k]=toupper(s[k]);

		if (s=="-CLUSTER") {
			i++;
			int n = atoi (argv[i]);
			if (n>0) {
				numberOfCluster = n;
			} else {
				usage();
			} // else
		} else {
			if (s=="-RECURSE") recurse=true;
			else {
				if (s=="-SELECT") select=true;
				else {
					if (i != argc-1) usage();
				} // else
			} // else
		} // else
	} // for
	


	SceneDetectionData scenes; 
	if (!recurse) {
		SceneDetection sceneDetect (filename, numberOfCluster, select);
		sceneDetect.process(scenes);
		cout << scenes << endl;

	} else {

		// read all MPEG files in directory
		vector<FileInfo> mpgFiles;
		FileList files(filename, true);    
		files.process("*.mpg", mpgFiles);

		// open mpeg files
	    vector<FileInfo>::iterator it;
		for (it = mpgFiles.begin(); it != mpgFiles.end(); it++) {
			cout << endl  << "processing file: "<< (*it).name << endl;
			SceneDetection sceneDetect ((*it).name, numberOfCluster, select);
			sceneDetect.process(scenes);
			cout << scenes << endl;

	} // for
		cout << endl << mpgFiles.size() << " files found." << endl;
	} // else

	cerr << endl << "time: " << (time(NULL)-startTime)  << " seconds" << endl;
	return 0;
} // main



