/*
   This file is part of the MoCA video library.
   Name: TextDetectionTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

  
	
	  \author    Stephan Kopf
	  
		Lehrstuhl fuer Praktische Informatik IV
		Universitaet Mannheim
		68131 Mannheim
		Germany
		email: kopf@informatik.uni-mannheim.de
		
		  (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
		  
*/


// C++ includes
#include <string>
#include <iostream>
using namespace std;

// MoCA includes
#include <TextRegionDetection.h>
#include <TextDetection.h>
#include <IImageStream.h>


const string textDetectionTestVersion = "1.0";
void usage()
{
	cerr << endl;
	cerr << "TextDetectionTest " << textDetectionTestVersion 
		<< "    21.07.2003   (c) Stephan Kopf, University of Mannheim" << endl;
	cerr << "---------------------------------------------------------------------------" << endl;
	cerr << endl;
	cerr << "Usage: TextDetectionTest input-file   " << endl;
	cerr << endl;
	cerr << "Description:" << endl;
	cerr << "An input file (video or image) is analysed and the text regions are stored." << endl;
	
	exit(1);
	return;
} // usage




void analyzeImage (string &srcFilename)
{

	// open image
	Image<uint8> img;
	ReadImage (srcFilename, img);

	// detect text regions
	TextRegionData data;
	TextRegionDetection region;
	region.process(img, data);

	// draw bounding ractangles	and store image
	region.draw (data, img);
	string fn = srcFilename.substr(0, srcFilename.size()-4) + "_out.png";
	WriteImage (img, fn);

	// show position of text regions 
	cout << data << endl;

#if 0
	TextDetectionData textData;
	bool storeFrame=true;
	TextDetection text(srcFilename, storeFrame);
	text.getTextRegion (img, region, textData);
	cout << text << endl;
#endif
	return;
} // analyzeImage


void analyzeMpgFile (string &srcFilename)
{
	VideoDecoder reader (srcFilename);
	VideoSize size = reader.getSize();

	Image<uint8> img;
	for (int32 i=0; i<size[DIM_Z]; i++) {

		// read image
		img=reader.getImage(i);

		// store frame
		string fn = FileHandling::getNumberedFilename("img",i,".jpg");
		WriteImage (img, fn);

		// analyze frame
		cerr << "\ranalyze frame " << i << "         " << endl;
		analyzeImage (fn);
	} // for

	return;
} // analyzeMpgFile 



int main(int argc, char* argv[])
{
	// analyse parameters
	// e.g., use file "movText-statScene.mpg" from our web page
	if (argc!=2) usage();
	
	string filename = argv[1];
	string extension="";
	FileHandling::getFileExtension (filename, extension);
	
	if (extension=="mpg") {
		analyzeMpgFile(filename);
	} else {
		analyzeImage(filename);
	} // else

	return 0;
} // main

