/*
   This file is part of the MoCA video library.
   Name: DerivationFilterTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#include <iostream>
#include <Image.h>
#include <IImageStream.h>
#include <OImageStream.h>
#include <DerivationFilter.h>
#include <MocaExceptions.h>

int main(int argc, char* argv[])
{
    if (argc != 4) {
        cerr << "Usage:   DerivationFilter src(pnm) dest(pnm) maskSize" << endl;
        cerr << "Example: DerivationFilter test.png out.png 3" << endl;        
        return 1;
    } // if

    Image<uint8> img;

    try {
        // open input filestream and read image
        IImageStream<uint8> iis(argv[1]);
        iis.read(img);
    } catch (MocaException e) {
        cout << "IImageStream " << e.what() << endl;
        exit (-1);
    } // try

    // manipulate image
    int maskSize = atoi(argv[3]);
    cout << "maskSize: " << maskSize<< endl;
    DerivationFilter<uint8> df (maskSize, DIR_XY);
    df.process(img);    
    
    try {
        // open output filestream and write image
        OImageStream<uint8> ois(argv[2]);
        ois.write(img);
    } catch (MocaException e) {
        cout << "OImageStream " << e.what() << endl;
        exit (-1);
    } // try

    return 0;
} // main



