/*
   This file is part of the MoCA video library.
   Name: FaceMaskTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf 

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

// C++ includes
#include <string>
#include <iostream>
using namespace std;

// Moca includes
#include <Image.h>
#include <IImageStream.h>
#include <OImageStream.h>
#include <FaceMaskFilter.h>
#include <FileList.h>

int main(int argc, char* argv[])
{
    if (argc != 3) {
        cerr << "Usage: FaceMaskTest directory(src) directory(dest) " << endl;
        return 1;
    } // if

    Image<uint8> src;
    Image<uint8> dest;

	// read all JPEG files in directory
	vector<FileInfo> files;
	FileList fl(argv[1], true);    
	fl.process("*.jpg", files);

	// open jpeg files
	vector<FileInfo>::iterator it;
	for (it = files.begin(); it != files.end(); it++) {

		try {
			
			string s = (*it).name;
			cout << endl  << "processing file: "<< s << endl;
			
			// open input filestream and read image
			IImageStream<uint8> iis(s);
			iis.read(src);
			
			// get face mask
			FaceMaskFilter fmf (FaceMaskFilter::FACE_MASK_COLOREYES);
			fmf.process(src, dest);
			
			// open output filestream and write image
			string d = argv[2];
			d += "CONTRAST_" + (*it).shortName;
			OImageStream<uint8> ois(d);
			ois.write(dest);
		} catch (MocaException e) {
			cout << "FaceMaskTest: " << e.what() << endl;
			exit (-1);
		} // try

	} // for
	cout << endl << files.size() << " files found." << endl;
	



    return 0;
} // main



