/*
   This file is part of the MoCA video library.
   Name: RotateImage.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#include <iostream>
using namespace std;

#include <Image.h>
#include <IImageStream.h>
#include <OImageStream.h>
#include <MocaExceptions.h>
#include <RotationFilter.h>

#include <CannyFilter.h>
#include <BinarizeFilter.h>
#include <InverseFilter.h>

int main(int argc, char* argv[])
{

    if (argc != 4) {
        cerr << "Usage:   RotateImage src1 src2 angle" << endl;
        return 1;
    } // if

    Image<uint8> img;
    cout << "read image: " << argv[1] << endl;
    try {
        // open input filestream and read image
        IImageStream<uint8> iis(argv[1]);
        iis.read(img);
    } catch (MocaException e) {
        cout << "IImageStream " << e.what() << endl;
        exit (-1);
    } // try

    ieee64 angle = atof(argv[3]);
    RotationFilter<uint8> rf(angle * RotationFilter<uint8>::PI/180.0);
    rf.process(img);

    cout  << "save image: " << argv[2] << endl;
    try {
        // open output filestream and write image
        OImageStream<uint8> ois(argv[2]);
        ois.write(img);
    } catch (MocaException e) {
        cout << "IImageStream " << e.what() << endl;
        exit (-1);
    } // try
    
    return 0;
} // main


