/*
   This file is part of the MoCA video library.
   Name: SvdTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include <SvdFit.h>
#include <iostream>

int main()
{
    const int ma = 8;
    const int ndata = 4*4 * 2;

    int x0[ndata/2] = {
        0, 8, 16, 24,
        0, 8, 16, 24,
        0, 8, 16, 24,
        0, 8, 16, 24
    };
    int y0[ndata/2] = {
        0, 0, 0, 0,
        8, 8, 8, 8,
        16, 16, 16, 16,
        24, 24, 24, 24
    };

    int x1[ndata/2] = {
        4, 12, 20, 28,
        4, 12, 20, 28,
        4, 12, 20, 28,
        4, 12, 20, 28
    };
    int y1[ndata/2] = {
        0, 0, 0, 0,
        8, 8, 8, 8,
        16, 16, 16, 16,
        24, 24, 24, 24
    };

    Matrix<ieee64> A(MatrixSize(ndata, ma));
    Vector<ieee64> b(ndata);

    int centerX = 16;
    int centerY = 16;

    int rowToFill = 0;
    for (int f = 0; f < ndata/2; f++) {
        int x    = x0[f] - centerX;
        int y    = centerY - y0[f];
        int xres = x1[f] - centerX;
        int yres = centerY - y1[f];

        // filling line for x coordinate
        A[rowToFill][0] = x;
        A[rowToFill][1] = y;
        A[rowToFill][2] = 1;
        A[rowToFill][3] = 0;
        A[rowToFill][4] = 0;
        A[rowToFill][5] = 0;
        A[rowToFill][6] = -x * xres;
        A[rowToFill][7] = -y * xres;
        b[rowToFill] = xres;
        rowToFill++;

        // filling line for y coordinate
        A[rowToFill][0] = 0;
        A[rowToFill][1] = 0;
        A[rowToFill][2] = 0;
        A[rowToFill][3] = x;
        A[rowToFill][4] = y;
        A[rowToFill][5] = 1;
        A[rowToFill][6] = -yres * x;
        A[rowToFill][7] = -y * yres;
        b[rowToFill] = yres;
        rowToFill++;
    }

    /*
    cout << "The matrix" << endl;
    cout << "[ ";
    for (int i = 1; i <= ndata; i++) {
        for (int j = 1; j <= 8; j++) {
            cout << A[i][j] << " ";
        }
        cout << ";" << endl;
    }

    cout << endl << "The result" << endl;
    for (int i = 1; i <= ndata; i++) {
        cout << b[i] << endl;
    }
    exit(1);
    */

    Vector<ieee64> a(ma);
    SvdFit svd;
    ieee64 chisq = svd.solve (A, a, b);

    for (int i = 0; i < a.getSize(); i++)
        cout << ((fabs(a[i]) > 0.000001) ? a[i] : 0)  << endl;

    cout << "error : " << sqrt(chisq) << endl;

    return 0;
}
