/*
   This file is part of the MoCA video library.
   Name: WavToMp2.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/

// C++ includes
#include <fstream>
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaSignal.h>
#include <IWaveAudio.h>
#include <OMpegAudio.h>


int main(int argc, char* argv[])
{

// read Wave file and store mp2 file

    if (argc != 3) {
        cerr << "read WAVE file and store MP2 file " << endl << endl;
        cerr << "Usage: " << endl;
		cerr << argv[0] << " src(wav)  dest(mp2) " << endl;
        return 1;
    } // if
	string srcFilename  = argv[1];
	string destFilename = argv[2];

	try {

		// read wave file
		cout << "read wav file..." << endl;
		IWaveAudio wav (srcFilename);
		WaveHeader header;
		header = wav.getHeader ();
		Signal<int16> sig;
		wav.read (sig);

		// store mp2 file
		int samplesPerSecond = header.getSamplesPerSec();
		int channels         = header.getChannels();
        
        {
            OMpegAudio writer (destFilename, samplesPerSecond, channels);
            writer.write(sig);
        }
        

    } catch (MocaException e) {
        cerr << "Mpeg2AudioTest  could not save mp2 file." << endl;
        cerr << e.what() << endl;
        exit (-1);
    } // try
	cout << endl;

	return 0;
} // main

