/*
   This file is part of the MoCA video library.
   Name: AudioCut.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef AUDIO_CUT_H
#define AUDIO_CUT_H

// C includes

// C++ includes
#include <iostream>
using namespace std;


// MoCA includes
#include <WaveHeader.h>
#include <MocaSignal.h>
#include <OTagStream.h>
#include <WriteMocaData.h>
#include <AudioEnergy.h>
#include <GaussFilter.h>
#include <WaveHeader.h>



const static string AUDIO_CUT_VERSION  ="Version-0.03";
const static string AUDIO_CUT_FILENAME ="audiocuts.idx";


struct AudioCutData
{
	AudioCutData () 
	{
		version = AUDIO_CUT_VERSION;
		filename = "";
		numberOfBlocks=0;
		numberOfCuts=0;
		minimumNumberOfCuts = 0;
		blockLengthMilliSeconds = 10;

	} 
    AudioCutData& operator=(const AudioCutData &other) {
		if (this != &other) {
			version           = other.version;
			filename          = other.filename;
			cuts              = other.cuts;
			numberOfBlocks    = other.numberOfBlocks;
			numberOfCuts      = other.numberOfCuts;
			minimumNumberOfCuts     = other.minimumNumberOfCuts;
			blockLengthMilliSeconds = other.blockLengthMilliSeconds;
		} // if
        return *this;
    } // operator=


	string version;
	string filename;
	int32 numberOfBlocks;
	int32 numberOfCuts;
	int32 minimumNumberOfCuts;
	int32 blockLengthMilliSeconds;


	// 0=no silence, 
	// 1=silent area 
	// 2=very silent area
	// values between 5..255: center of silent area
	Signal<uint8> cuts;
};

std::ostream& operator<< (ostream &, const AudioCutData&);
OTagStream& operator<< (OTagStream&, const AudioCutData&);
ITagStream& operator>> (ITagStream&, AudioCutData&);





/*
	Constructor: initializes variables
	
	process(): 
		if status file does not exist:
		   calculate the audio cuts from the WAV file, save status file
		else:
		   read audio cuts from status file
*/
class AudioCut
{
public:

	AudioCut(const string &pathFilename, int32 minimumNumberOfCuts, int32 blockLengthMilliSeconds);
	~AudioCut() {};

	void process (AudioCutData&);

private:
	
	void readValues (AudioCutData& data);
	string pathFilename;
	int32 blockLengthMilliSeconds;
	int32 minimumNumberOfCuts;
};

#endif
