/*
    ContourParametrization.cpp     parametrize a contour
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ContourParametrization.h>


/*!
	static variables
*/
//const uint16 ContourParametrization::DEFAULT_CONTOUR_SIZE = 200;
const uint16 ContourParametrization::DEFAULT_CONTOUR_SIZE = 200;


/*!
  Constructor
 */

ContourParametrization::
ContourParametrization (uint16 numberContourPixel)
{
    this->numberContourPixel = numberContourPixel;
} // ContourParametrization



/*!
  Destructor
 */

ContourParametrization::
~ContourParametrization ()
{
} // ~ContourParametrization



ieee64
ContourParametrization::
getDistance(ImageEdgePosition &p1, ImageEdgePosition &p2) {

	int d1 = labs(p1[DIM_X]-p2[DIM_X]);
	int d2 = labs(p1[DIM_Y]-p2[DIM_Y]);
	if (d1 == 0) {
		if (d2 == 0) {
			return 0;
		} else {
			return d2;
		} // if
	} else {
		if (d2 == 0) {
			return d1;
		} // if
	}
	return sqrt ((double)(d1+d2));

} // getDistance


/*!
  Process
 */

void
ContourParametrization::
process(vector<ImageEdgePosition> &src, vector<ImagePosition> &dest)
{
	// calculate length of contour
	ieee64 length = 0;

	vector<ImageEdgePosition>::iterator lastIt;
	vector<ImageEdgePosition>::iterator it;

	if (src.begin() != src.end()) {
		lastIt  = src.begin();
		it      = src.end();
		it--;
		length += getDistance (*lastIt, *it);

        vector<ImageEdgePosition>::iterator startIt = src.begin();
        startIt++;
	    for (it = startIt; it != src.end(); it++) {
		    length += getDistance (*it, *lastIt);
			lastIt = it;
		} // for
		
		//cout << "LENGTH: " << length << endl;

		// push contour pixel in vector
		ieee64 stepSize = length / (ieee64) numberContourPixel;
		ieee64 stepPosition = stepSize;
		length = 0;
		lastIt  = src.begin();
		it      = src.begin();
		it++;
		dest.push_back (ImagePosition((*lastIt)[DIM_X],(*lastIt)[DIM_Y]));
        //        cout << "FOR" << endl;
        //        cout << stepPosition << "  " << length << "  " <<endl;
		for (int i=1; i< numberContourPixel; i++) {
			while (length < stepPosition) {
				length += getDistance (*it, *lastIt);
				it++;
				lastIt++;
                if (it==src.end()) {
                    it = src.begin();
                    lastIt = src.end();
                    lastIt--;
                }
                if (lastIt==src.end()) {
                    lastIt = src.begin();
                    it = src.begin();
                    it++;
                }

			} // while
            //            cout << (*it)[DIM_X] << "  "<< (*it)[DIM_Y]<< endl;
            dest.push_back (ImagePosition((*lastIt)[DIM_X],(*lastIt)[DIM_Y]));
			stepPosition += stepSize;
		} // for
	} // if
	return;
} // process
