/*
   This file is part of the MoCA video library.
   Name: CssImage.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#ifndef CSS_IMAGE_H
#define CSS_IMAGE_H

// C++ includes
#include <vector>
#include <sstream>
using namespace std;

// MoCA includes
#include <Image.h>
#include <ImagePosition.h>
#include <MocaSignal.h>
#include <GaussFilter.h>
#include <DrawLine.h>
#include <OImageStream.h>
#include <ImageEdgePosition.h>
#include <ContourEdgeList.h>
#include <ContourParametrization.h>
#include <DrawDot.h>
#include <DrawCross.h>
#include <DrawCircle.h>
#include <RegionFeature.h>
#include <DilationFilter.h>




class CssImage
{
public:
	// constructor
    CssImage (ieee64 percentNoise = 0.3, uint8 gaussRadius=1, uint16 maxSigmaSize=6000);
    ~CssImage () {};

	// main routine that generates a CSS image
	//   src: definition of the contour of the object (vector or "signal" of contour points) 
	//   img: CSS image
	void process(const vector<ImagePosition> &src, Image<uint8> &img);
	void process(const Signal<int16> &src, Image<uint8> &img);


	// main routine that generates a CSS image
	//   src: input image
	//   img: CSS image
	void process(const Image<uint8> &src, Image<uint8> &img);


	// main routine that generates two CSS images, the standard image and an "inverted" image
	//   src: input image
	//   stdCss: standard CSS image
	//   invCss: inverted CSS image
	void process(const Image<uint8> &src, Image<uint8> &stdCss, Image<uint8> &invCss, Image<uint8> &debugContour);

	// main routine that generates a CSS image
	//   src: definition of the contour of the object (vector or "signal" of contour points) 
	//   stdCss: standard CSS image
	//   invCss: inverted CSS image
	void process(const vector<ImagePosition> &src, Image<uint8> &stdCss, Image<uint8> &invCss, Image<uint8> &debugContour);


	// calculate inverted contour
	void calcInvertedContour (Signal<int16> &ip);
	void resampleInvertedContour (Signal<int16> &ip);


private:
	// functions
	void fillHoles(Image<uint8> &img);
	void mark(int x1,int y1,int x2,int y2,int value, Image<uint8> &img, Signal<bool> &isConcave);
    inline ieee64 abs (ieee64 x);
	uint8 findNewValue(int x, uint8& minValue, uint8& maxValue, Signal<bool> &isConcave);
	void getContour(const Image<uint8> &img, vector<ImagePosition> &contour); 

	// variables
	uint8  gaussRadius;
	uint16 maxSigmaSize;
	ieee64 percentNoise;

	// variables for the inverted css approach
	int32 centerX;
	int32 centerY;
	int32 radius;

	// static variables
	const static uint8 WHITE;
	const static uint8 BLACK;
	const static uint8 GREY;
};


#endif







