/*
   This file is part of the MoCA video library.
   Name: CssVideoObject.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef CSS_VIDEO_OBJECTS_H
#define CSS_VIDEO_OBJECTS_H


// C++ includes
#include <iomanip>
#include <sstream>
#include <iostream>
using namespace std;


// MoCA includes
#include <WriteMocaData.h>
#include <CssVideoAnalysis.h>
#include <CssDatabase.h>


const static string CSS_VIDEO_OBJECT_VERSION  ="Version-0.03";
const static string CSS_VIDEO_OBJECT_FILENAME ="css.idx";


struct CssVideoObjectItem
{
	CssVideoObjectItem ()
	{
		frame  = -1;
		shot   = -1;
		databaseRef  = -1;
		databaseDist = 10000000;
		px1 = 0;
		px2 = 0;
		py1 = 0;
		py2 = 0;
	}

	CssVideoObjectItem (CssMaximumData &cmList, int32 frame, int32 shot, int32 px1, int32 px2, int32 py1, int32 py2)
	{
		data = cmList;
		this->frame  = frame;
		this->shot   = shot;
		databaseRef  = -1;
		databaseDist = 10000000;
		this->px1 = px1;
		this->px2 = px2;
		this->py1 = py1;
		this->py2 = py2;
	}

    CssVideoObjectItem& operator=(const CssVideoObjectItem &other) {
		if (this != &other) {
			data         = other.data;
			frame        = other.frame;
			shot         = other.shot;
			databaseRef  = other.databaseRef;
			databaseDist = other.databaseDist;
			px1 = other.px1;
			px2 = other.px2;
			py1 = other.py1;
			py2 = other.py2;
		} // if
        return *this;
    } // operator=


	CssMaximumData data;
	int32 frame;
	int32 shot;
	int32 databaseRef;
	int32 databaseDist;
	int32 px1;
	int32 px2;
	int32 py1;
	int32 py2;
};


struct CssVideoObjectData
{
	CssVideoObjectData () 
	{
		version = CSS_VIDEO_OBJECT_VERSION;
		this->filename = "";;
		numberOfItems = 0;
		data.clear();
	} 
    CssVideoObjectData& operator=(const CssVideoObjectData &other) {
		if (this != &other) {
			version         = other.version;
			filename        = other.filename;
			numberOfItems   = other.numberOfItems;
			data            = other.data;
		} // if
        return *this;
    } // operator=

	string version;
	string filename;
	int32  numberOfItems;
	vector<CssVideoObjectItem> data;
};

std::ostream& operator<< (ostream &, const CssVideoObjectData&);
OTagStream& operator<< (OTagStream&, const CssVideoObjectData&);
ITagStream& operator>> (ITagStream&, CssVideoObjectData&);





/*
	Constructor: initializes variables
	
	process(): 
		if status file does not exist:
		   calculate the action in mpeg-file, save status file
		else:
		   read action data from status file
*/
class CssVideoObject
{
public:

	CssVideoObject(const string &fullName);
	~CssVideoObject() {};

	void process (CssVideoObjectData&/*, bool update*/);

private:
//	void updateValues(CssVideoObjectData& data);
	void readValues (CssVideoObjectData& data);

	string pathFilename;
};

#endif
