/*! \file
 * 
   \author    Manuel Knaus

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/


#ifndef HISTORIC_VIDEO_OPTIMIZATION_H
#define HISTORIC_VIDEO_OPTIMIZATION_H


// C++ includes
#include <iostream>
#include <sstream>
#include <stdlib.h>
using namespace std;


// MoCA includes
#include <Image.h>
#include <OImageStream.h>
#include <VideoDecoder.h>
#include <ScaleFilter.h>
#include <VideoEncoder.h>
#include <DrawLine.h>
#include <LuminanceFilter.h>
#include <ContrastFilter.h>
#include  <ImageStatistic.h>
#include <DrawDot.h>
#include <CutDetectionFeature.h>
#include <CameraModel.h>




/*
	Constructor: initializes variables
	
	process(): 

*/
class HistoricVideoOptimization
{
  public:
    
    /*
	 HistoricVideoOptimization(int yCounter[1200], int yMaxPos, 
                               const bool TESTRUN, bool horizontalLineDetected,
                               int maxCorrFactorOfActualCutDirectionX,
                               int maxCorrFactorOfActualCutDirectionY,
                               int minCorrFactorOfActualCutDirectionX,
                               int minCorrFactorOfActualCutDirectionY,
                               const double THRESBIGMOVEMENT,
                               bool doLineDetection, bool doShakingDetection,
                               bool doLumContrastCorrection );
    */
    HistoricVideoOptimization();

    /* Main Method of Historic Video Optimization Process. A given video is 
    * will be corrected and the output of the video file will be in the same
    * directory with an ending "_new.avi"
    * input: bools, wheather single corrections have to be done
    * output: corrected video file
    * */   	
    void process(string &inputFileName, bool doLineDetection, 
                                        bool doShakingDetection,
                                        bool doLumContrastCorrection);


  private:

    bool checkLumImbalance(int lumMax, int lumMin);

    int findMaxLum(ImageStatisticData &statdat, int lumMax);

    int findMinLum(ImageStatisticData &statdat, int lumMin);

    int findNextCut(int actualCut, CutDetectionData &cutdata, int vidsize) ; 
 
    bool checkOverbright(int lumMax, int lumMin);

    bool checkOverdark(int lumMax, int lumMin);

    int computeLumChangeRate(int actLum);

    int computeContrastChangeRate(int lumMax, int lumMin, int actLum, bool overbright, 
                                  bool overdark, bool changeLum, int lumChangeRate);

    int averageValue(int a, int b, double factor);

    bool checkPixel(int32 posAct, int32 posLeft, int32 posRight, int32 pos2Left, 
                    int32 pos2Right, int32 pos2Above, int32 pos2Below);

    bool check4BlackPixel(int32 posAct, int32 pos3Left, int32 pos3Right, 
                          int32 pos2Above, int32 pos2Below);

    void presearchHorizontalLine(Image<uint8> &img, VideoSize &size);

    Image<uint8> findHorizontalLine(Image<uint8> &img, VideoSize &size);

    int yMax();

    void findMaxCorrFactors(int actualCut, int nextCut, int middleShakingPos,
                            CameraModelData &cmData);

    int preprocessAntiShaking(int actualCut, int nextCut, bool directionX, 
                              CameraModelData &cmData);

    Image<uint8> processAntiShaking(int i, bool directionX, CameraModelData &cmData, 
                                    Image<uint8> &img, int middleShakingPos, 
                                    VideoSize &size) ;

  /* Global static variables and constants*/
  int yCounter[1200]; //y Counter for line detection, test value 1200
  int yMaxPos ;  //needed for line detetion
  bool TESTRUN ; //true, if its a test
  bool horizontalLineDetected ; //only for shell output needed
  int maxCorrFactorOfActualCutDirectionX ;
  int maxCorrFactorOfActualCutDirectionY ;
  int minCorrFactorOfActualCutDirectionX;
  int minCorrFactorOfActualCutDirectionY ;
  double THRESBIGMOVEMENT;  //15 //20
  bool doLineDetection ; //possibility to camcel a special process
  bool doShakingDetection ;
  bool doLumContrastCorrection ;                                
                                    
};

#endif
