/*
    ProjectionProfile.cpp     detect text
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <ProjectionProfile.h>


void
ProjectionProfile::process (const Image<uint8> &img, Signal<int32> &horizontalProfile, Signal<int32> &verticalProfile)
{
	ImageSize s=img.getSize();
	SignalSize hs (s[DIM_X]);
	SignalSize vs (s[DIM_Y]);

	horizontalProfile.resize(hs);
	verticalProfile.resize(vs);
	horizontalProfile.clear(0);
	verticalProfile.clear(0);

	// fill profile
	switch (mode) {
	case PROJECTION_PROFILE_DIFF:
		for (int32 b=0; b<s[DIM_BAND]; b++) {
			for (int32 y=0; y<s[DIM_Y]; y++) {
				int32 v=0;
				for (int32 x=0; x<s[DIM_X]-1; x++) {
					v += labs (img(x,y,0) - img(x+1,y,0));
				} // for
				verticalProfile(y) += v;
			} // for
		} // for
		for (int32 b=0; b<s[DIM_BAND]; b++) {
			for (int32 x=0; x<s[DIM_X]; x++) {
				int32 v=0;
				for (int32 y=0; y<s[DIM_Y]-1; y++) {
					v += labs (img(x,y,0) - img(x,y+1,0));
				} // for
				horizontalProfile(x) += v;
			} // for
		} // for
		break;
	case PROJECTION_PROFILE_ABS:
		for (int32 b=0; b<s[DIM_BAND]; b++) {
			for (int32 y=0; y<s[DIM_Y]; y++) {
				int32 v=0;
				for (int32 x=0; x<s[DIM_X]-1; x++) {
					v += img(x,y,0);
				} // for
				verticalProfile(y) += v;
			} // for
		} // for
		for (int32 b=0; b<s[DIM_BAND]; b++) {
			for (int32 x=0; x<s[DIM_X]; x++) {
				int32 v=0;
				for (int32 y=0; y<s[DIM_Y]-1; y++) {
					v += img(x,y,0);
				} // for
				horizontalProfile(x) += v;
			} // for
		} // for
		break;
	default:
		cerr << "ProjectionProfile::process: unknown mode!" << endl;
		exit(1);
	}

	return;
}


void
ProjectionProfile::process (const Image<uint8> &img, 
		                    Signal<int32> &horizontalProfile, Signal<int32> &verticalProfile,
				            Image<uint8> &dest, int32 addWidth, int32 addHeight, Color c)
{
	process (img, horizontalProfile, verticalProfile);

	ImageSize s=img.getSize();
	ImageSize destSize =s;
	destSize[DIM_X]+=addWidth;
	destSize[DIM_Y]+=addHeight;
	dest.resize(destSize);
	dest.clear(255);
	dest.copy (img);

	// modify horizontal values
	Signal<int32> hp = horizontalProfile;
	{
		ieee64 hMax=0;
		ieee64 hMin=10e10;
		for (int32 i=0; i<s[DIM_X]; i++) {
			if (hMax<hp(i)) hMax=hp(i);
			if (hMin>hp(i)) hMin=hp(i);
		} // for
		hMax -= hMin;
		hMax = addHeight / hMax;
		for (int32 i=0; i<s[DIM_X]; i++) {
			hp(i) = (int32)(hMax * (hp(i) - hMin)); 
		} // for
	}
	
	// modify vertical values
	Signal<int32> vp = verticalProfile;
	{
		ieee64 vMax=0;
		ieee64 vMin=10e10;
		for (int32 i=0; i<s[DIM_Y]; i++) {
			if (vMax<vp(i)) vMax=vp(i);
			if (vMin>vp(i)) vMin=vp(i);
		} // for
		vMax -= vMin;
		vMax = addWidth/ vMax;
		for (int32 i=0; i<s[DIM_Y]; i++) {
			vp(i) = (int32)(vMax * (vp(i) - vMin)); 
		} // for
	}

	// draw profile
	DrawLine line (c);
	for (int32 i=0; i<s[DIM_Y]; i++) {
		ImagePosition p0 (s[DIM_X], i);
		ImagePosition p1 (s[DIM_X]+vp(i), i);
		line.process(p0,p1,dest);
	} // for
	for (int32 i=0; i<s[DIM_X]; i++) {
		ImagePosition p0 (i, s[DIM_Y]);
		ImagePosition p1 (i, s[DIM_Y]+hp(i));
		line.process(p0,p1,dest);
	} // for

	return;
}

