/*
   This file is part of the MoCA video library.
   Name: StructureMatching.h
   Copyright (C) 2005 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef STRUCTURE_MATCHING_FEATURE_H
#define STRUCTURE_MATCHING_FEATURE_H

// C++ includes
#include <vector>

// MoCA includes
#include <Image.h>
#include <OImageStream.h>
#include <IImageStream.h>
#include <CropFilter.h>
#include <CannyFilter.h>


//! Identify position of similar structures in the image (structure is defined by input image)
/*!
  This class will identify the position of similar structures in the image
*/

struct StructureMatchingFeatureItem;

struct StructureMatchingFeatureItem 
{
	StructureMatchingFeatureItem ()
	{
		x=0;
		y=0;
		diff=0;
	}

    StructureMatchingFeatureItem& operator=(const StructureMatchingFeatureItem &other) {
		if (this != &other) {
			x    = other.x;
			y    = other.y;
			diff = other.diff;
		} // if
        return *this;
    } // operator=

#if 0
	bool operator< (const StructureMatchingFeatureItem& f1, const StructureMatchingFeatureItem& f2) {
		return (f1.diff < f2.diff);
	} // operator<
#endif

	int32 x, y;
	ieee64 diff;
};
std::ostream& operator<< (ostream &, const StructureMatchingFeatureItem&);


struct StructureMatchingFeatureData 
{
	StructureMatchingFeatureData (){}

    StructureMatchingFeatureData& operator=(const StructureMatchingFeatureData &other) {
		if (this != &other) {
			region = other.region;
		} // if
        return *this;
    } // operator=

	list<StructureMatchingFeatureItem> region;
};


class StructureMatchingFeature
{
public:

	StructureMatchingFeature (ieee64 th) 
	{
		this->th     = th;
	}

	~StructureMatchingFeature () {};

	// analyze image
	// refImg: reference image defines the structure
	// img   : image to analyse
	// data  : image positions
	void process (const Image<uint8> &refImg, const Image<uint8> &img, StructureMatchingFeatureData &data);


private:
	ieee64 th;           // only image positions are found where the similarity does not exceed the threshold
};
#endif
