/*
    TextRegionDetection.cpp     detect text regions
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <TextRegionDetection.h>

std::ostream& operator<< (ostream &os, const TextRegionRect &data) 
{
	os << "minX       : " << data.minX << endl;
	os << "maxX       : " << data.maxX << endl;
	os << "minY       : " << data.minY << endl;
	os << "maxY       : " << data.maxY << endl;
	os << "reliability: " << data.minX << endl;
	if (data.analysed) os << "analyzed   : true " << endl;
	else os << "analyzed   : false " << endl;
	if (data.delData) os << "deleted    : true " << endl;
	else os << "deleted    : false " << endl;

	return os;
} // operator <<



std::ostream& operator<< (ostream &os, const TextRegionData &data) 
{
	if (data.size == 0) {
		os << "no text region found" << endl;
	} else {
		os << "size: " << data.size << endl;
		for (int32 i=0; i<data.box.size(); i++)
			os << data.box[i] << endl;
	} // else
	return os;
} // operator <<



void 
TextRegionDetection::getEdgeImage (Image<uint8> &src, Image<int32> &dest)
{
const int thEdgeStrength = 50;
    ImageSize size = src.getSize();
    size[DIM_BAND] = 1;
    dest.resize(size);
    dest.clear(0);
    for (int32 b=0; b<size[DIM_BAND]; b++) {
        for (int32 y=1; y<size[DIM_Y]-1; y++) {
            for (int32 x=1; x<size[DIM_X]-1; x++) {
                int32 v = (int32) MAX (ABS (src(x,y,b)-src(x+1,y,b)), ABS (src(x,y,b)-src(x+2,y,b)));
                if (v>thEdgeStrength) {
                    if (dest(x,y)<v) dest(x,y)=v;
                } // if
            } // for
        } // for
    } // for
    return;
}


void 
TextRegionDetection::draw (TextRegionData& data, Image<uint8> &img) 
{
    DrawRectangle draw (Color::RED);
    for (int32 i=0; i<data.box.size(); i++) {
        // draw rectangle
        ImagePosition p0 (data.box[i].minX, data.box[i].minY);
        ImagePosition p1 (data.box[i].maxX, data.box[i].maxY);
        draw.process (p0, p1, img);
        // draw rectangle
        p0[DIM_X]--;
        p0[DIM_Y]--;
        p1[DIM_X]++;
        p1[DIM_Y]++;
        draw.process (p0, p1, img);
    } // for
    //cout << data.box.size() << " text regions found" << endl;
    return;
} // draw


void 

TextRegionDetection::store (Signal<int32> &line, TextRegionRect &r, string &fn) 

{
	int32 my = line.getSize()[DIM_X];
	ImageSize size (200,  my, 3);
	Image<uint8> out (size);
	out.clear(255);


	// store projection file
	int32 maxV=0;
	for (int32 y=0; y<my; y++) {
		if (maxV<line(y)) maxV=line(y);
	} // for
	
	DrawLine draw (Color::GRAY);
	ImagePosition p0 (5,0);
	ImagePosition p1 (5,0);
	for (int32 y=0; y<my; y++) {
		p0[DIM_Y] = y;
		p1[DIM_Y] = y;
		p1[DIM_X] = (200 * line(y) / maxV+5);
		if (p1[DIM_X] >= 200) p1[DIM_X] = 200-1;
		draw.process (p0, p1, out);
	} // for+

	DrawLine draw1 (Color::RED);
	p0[DIM_Y] = r.minY;
	p1[DIM_Y] = r.minY;
	p0[DIM_X] = 0;
	p1[DIM_X] = 5;
	draw1.process (p0, p1, out);
	p0[DIM_Y] = r.maxY;
	p1[DIM_Y] = r.maxY;
	draw1.process (p0, p1, out);
	
	p0[DIM_X] = 0;
	p1[DIM_X] = 0;
	p0[DIM_Y] = r.minY;
	draw1.process (p0, p1, out);


	WriteImage (out, fn);

	return;
} // store



void 
TextRegionDetection::storeDebug (const Image<uint8> &srcOrig, const Image<int32> &src, const string& fnNr) 
{
#if 0

	string fnPath = "D:/temp/";
	ImageSize s = src.getSize();
	Image<uint8> tmp(s);
	for (int32 b=0; b<s[DIM_BAND]; b++) {
		for (int32 y=0; y<s[DIM_Y]; y++) {
			for (int32 x=0; x<s[DIM_X]; x++) {
				tmp(x,y,b) = (uint8) src(x,y,b);
			} // for
		} // for
	} // for
	string fn2 = fnPath + "img" + fnNr + ".png";
	WriteImage (tmp,fn2);

	// mark regions in color image
	const int vv0=-120;
	const int vv1=50;
	tmp = srcOrig;
	LuminanceFilter lf (vv0);
	lf.process (tmp);
	for (int32 y=0; y<s[DIM_Y]; y++) {
		for (int32 x=0; x<s[DIM_X]; x++) {
			if (src(x,y) == 255) {

				for (int32 dx=0; dx<16; dx++) {

					int32 v0=srcOrig(16*x+dx,y,0);
					int32 v1=srcOrig(16*x+dx,y,1);
					int32 v2=srcOrig(16*x+dx,y,2);
					v0+=vv1;
					v1+=vv1;
					v2+=vv1;
					if (v0>255) v0=255;
					if (v1>255) v1=255;
					if (v2>255) v2=255;
					tmp(16*x+dx,y,0) = (uint8) v0;
					tmp(16*x+dx,y,1) = (uint8) v1;
					tmp(16*x+dx,y,2) = (uint8) v2;
				} // for

			} // if
		} // for
	} // for

	fn2 = fnPath + "img" + fnNr + "_color.png";
	WriteImage (tmp,fn2);

#endif
	return;
}


void 
TextRegionDetection::debugStoreTextRegions (const Image<uint8> &srcOrig, TextRegionData& result, const string &fn1) 
{
#if 0

	ImageSize s = srcOrig.getSize();
	Image<uint8> tmp = srcOrig;

	DrawRectangle rect (Color::RED, 3);
	for (int32 i=0; i<result.box.size(); i++) {
		TextRegionRect r = result.box[i];
		if (!r.delData) {
			if (r.minX != -1) {
				// debug for testing
				//if (r.minX>17 && r.maxX<838 && r.maxX>690 && r.maxY-r.minY>9 || r.minY>434)
				rect.process(ImagePosition (r.minX, r.minY), ImagePosition (r.maxX, r.maxY), tmp);
			} // if
		} // if
	} // for

	string fn = "D:/temp/" + fn1;;
	WriteImage (tmp,fn);
#endif

	return;
}


void 
TextRegionDetection::process(const Image<uint8> &input, TextRegionData& result) 
{


	TextRegionData data;

	// convert to YUV
	Image<uint8> src;
	VideoCodec srcMode = CODEC_RGB;
	VideoCodec destMode= CODEC_YUV444;
	ConvertColorSpace::process(srcMode, destMode, input, src);
	ImageSize size = src.getSize();


	// analyse original image
	Image<int32> edges;
	getEdgeImage (src, edges);


	// scale image and analyse each scaled image, integrate the results to edges 
	Image<uint8> imgScaled;
	Image<int32> edgesScaled;
	ImageSize scaledSize = size;
	ScaleFilter<uint8> scale;
	int factor = 2;
	Image<uint8> img = src;

	const int minTextHeight= 15;
	const int minTextWidth = 40;
	while ((scaledSize[DIM_X] > 2*minTextWidth) && (scaledSize[DIM_Y] > 2*minTextHeight)) {

		// reduce image size by factor 2
		scale.reduce (img, imgScaled, factor, factor);
		scaledSize = imgScaled.getSize();

		// get edges
		getEdgeImage (imgScaled, edgesScaled);

		// add edgesScaled to edges
		for (int32 y=0; y<size[DIM_Y]; y++) {
			for (int32 x=0; x<size[DIM_X]; x++) {
				edges(x,y) += edgesScaled (x/factor, y/factor);
			} // for
		} // for
		factor *=2;

	} // while


	// initialize 16x1 regions
	ImageSize regionSize = size;
	regionSize[DIM_X] = regionSize[DIM_X]/16+1;
	regionSize[DIM_BAND] = 1;
	Image<int32> region (regionSize);
	region.clear(0);

	const int32 thLow    = 1000;
	const int32 thHigh   = 3000;
	for (int32 y=0; y<regionSize[DIM_Y]; y++) {
		for (int32 x=0; x<regionSize[DIM_X]; x++) {
			for (int32 i=0; i<16; i++) {
				region (x,y) += edges(x*16+i,y);
			} // for
			if (region (x,y) < thLow) {
				region (x,y)=0;
			} else {
				if (region (x,y) > thHigh) {
					region (x,y)=255;
				} else {
					region (x,y)=128;
				} // else
			} // else
		} // for
	} // for
	storeDebug (input, region, "01") ;


	// increase text regions (med)
	bool modified=true;
	while (modified) {
		modified = false;
		for (int32 y=1; y<regionSize[DIM_Y]-1; y++) {
			for (int32 x=1; x<regionSize[DIM_X]-1; x++) {
				if (region (x,y) ==128) {
					if ((region (x-1,y-1)==255) || (region (x-1,y)==255) || (region (x-1,y+1)==255) ||
						(region (x+1,y-1)==255) || (region (x+1,y)==255) || (region (x+1,y+1)==255)) {
						region (x,y) = 255;
						modified = true;
					} // if
				} // if
			} // for
		} // for
	} // while


	// remove other regions
	for (int32 y=0; y<regionSize[DIM_Y]; y++) {
		for (int32 x=0; x<regionSize[DIM_X]; x++) {
			if (region (x,y) != 255) 
				region (x,y) = 0;
		} // for
	} // for


	// remove small regions
	for (int32 y=0; y<regionSize[DIM_Y]; y++) {
		for (int32 x=1; x<regionSize[DIM_X]-1; x++) {
			if ((region (x-1,y) == 0) && (region (x+1,y) == 0)) {
				region (x,y) = 0;
			} // if
			if ((region (x-1,y) == 0) && (region (x+2,y) == 0)) {
				region (x,y) = 0;
				region (x+1,y) = 0;
			} // if
		} // for
	} // for
	storeDebug (input, region, "02") ;



	// smooth regions
	ClosingFilter<int32> cf;
	cf.process (region);
	storeDebug (input, region, "03") ;


	// set boder to non-text
	for (int32 y=0; y<regionSize[DIM_Y]; y++) {
		region(0,y) = 0;
		region(regionSize[DIM_X]-1,y) = 0;
	} // for
	for (int32 x=0; x<regionSize[DIM_X]; x++) {
		region(x,0) = 0;
		region(x,regionSize[DIM_Y]-1) = 0;
	} // for
	storeDebug (input, region, "04") ;



	// detect regions
	RegionFeature<int32> rf;
	Image<uint32> area;
	rf.process(region, area);


	// initialize text regions
	data.size=0;
	data.box.clear();
	for (int32 y=0; y<regionSize[DIM_Y]; y++) {
		for (int32 x=0; x<regionSize[DIM_X]; x++) {
			if (region(x,y)== 255) {
				int32 x0 = x-1;
				int32 x1 = x+1;
				int32 y0 = y-1;
				int32 y1 = y+1;
				uint32 v = area(x,y);

				// get y0
				bool found = true;
				while (found) {
					found = false;
					for (int32 i=0; i<regionSize[DIM_X]; i++) {
						if (v == area(i,y0)) {
							y0--;
							found=true;
							break;
						} // if
					} // for
				} // while
				y0++;

				// get y1
				found = true;
				while (found) {
					found = false;
					for (int32 i=0; i<regionSize[DIM_X]; i++) {
						if (v == area(i,y1)) {
							y1++;
							found=true;
							break;
						} // if
					} // for
				} // while
				y1--;

				// get x0
				found = true;
				while (found) {
					found = false;
					for (int32 i=0; i<regionSize[DIM_Y]; i++) {
						if (v == area(x0,i)) {
							x0--;
							found=true;
							break;
						} // if
					} // for
				} // while
				x0++;

				// get x1
				found = true;
				while (found) {
					found = false;
					for (int32 i=0; i<regionSize[DIM_Y]; i++) {
						if (v == area(x1,i)) {
							x1++;
							found=true;
							break;
						} // if
					} // for
				} // while
				x1--;

				// store text box
				TextRegionRect rect;
				rect.minX = 16*x0;
				rect.maxX = 16*x1;
				rect.minY = y0;
				rect.maxY = y1;
				rect.rel  = 0.5;
				data.box.push_back(rect);

				for (int32 j=y0; j<=y1; j++) {
					for (int32 i=x0; i<=x1; i++) {
						if (area(i,j) == v) {
							region (i,j) = 0;
						} // if
					} // for
				} // for
				

			} // if
		} // for
	} // for
	debugStoreTextRegions (input, data, "regions01.png");


	// initialize dx and dy
	ImageSize dxSize = size;
	dxSize[DIM_BAND] = 1;
	Image<uint8> dx (dxSize);
	Image<uint8> dy (dxSize);
	dx.clear(0);
	dy.clear(0);
	for (int32 y=1; y<size[DIM_Y]-1; y++) {
		for (int32 x=1; x<size[DIM_X]-1; x++) {
			int32 v = (int32) MAX(ABS(src(x,y)-src(x+1,y)), ABS(src(x-1,y)-src(x+1,y)));
			if (v>50) dx (x,y) = (uint8) v;
			v = (int32) MAX(ABS(src(x,y)-src(x,y+1)), ABS(src(x,y-1)-src(x,y+1)));
			if (v>50) dy (x,y) = (uint8) v;
		} // for
	} // for


	// text size thresholds
	const int32 thTextWidth = 24;
	const int32 thTextHeight = 6;


	// get exact vertical position of text lines
	SignalSize lineSize (size[DIM_Y]);
	Signal<int32> lineDx (lineSize);
	Signal<int32> lineDy (lineSize);
	bool rectAnalysed = false;
	int32 maxRegionInc = 10;
	while (!rectAnalysed) {
		rectAnalysed = true;
		maxRegionInc-=2;
		if (maxRegionInc<0) maxRegionInc=0;


		// delete very small text boxes
		for (int32 i=0; i<data.box.size(); i++) {
			if ((!data.box[i].analysed) && (!data.box[i].delData)) {
				int32 v = data.box[i].maxX - data.box[i].minX;
				if (v<thTextWidth) {
					data.box[i].delData = true;
				} // if
				v = data.box[i].maxY - data.box[i].minY;
				if (v<thTextHeight) {
					data.box[i].delData = true;
				} // if 
			} // if
		} // for


		int32 n = data.box.size();
		for (int32 i=0; i<n; i++) {

			if ((!data.box[i].analysed) && (!data.box[i].delData)) {

				// initialize projection
				lineDx.clear(0);
				lineDy.clear(0);
				TextRegionRect r = data.box[i];
				r.minY -=maxRegionInc; if (r.minY < 0) r.minY=0;
				r.maxY +=maxRegionInc; if (r.minY >= size[DIM_Y]) r.minY=size[DIM_Y]-1;
				for (int32 y=r.minY; y<r.maxY; y++) {
					for (int32 x=r.minX; x<r.maxX; x++) {
						lineDx (y) += dx (x,y);
						lineDy (y) += dy (x,y);
					} // for
				} // for
				

				// find minimum, and reduce by minimum
				int32 minV = lineDx (r.minY);
				for (int32 y=r.minY; y<r.maxY; y++) {
					if (minV > lineDx(y)) minV = lineDx(y);
				} // for
				for (int32 y=r.minY; y<r.maxY; y++) {
					lineDx(y) -= minV;
				} // for
				minV = lineDy (r.minY);
				for (int32 y=r.minY; y<r.maxY; y++) {
					if (minV > lineDy(y)) minV = lineDy(y);
				} // for
				for (int32 y=r.minY; y<r.maxY; y++) {
					lineDy(y) -= minV;
				} // for
		

				// set all values less than 10% max to zero
				int32 maxV = lineDx (r.minY);
				for (int32 y=r.minY; y<r.maxY; y++) {
					if (maxV < lineDx(y)) maxV = lineDx(y);
				} // for
				maxV /=10;
				for (int32 y=r.minY; y<r.maxY; y++) {
					if (lineDx(y)<maxV) lineDx(y)=0;
				} // for
			
				// update minY and maxY
				//while ((r.minY < r.maxY) && (lineDy (r.minY) == 0)) r.minY++; 
				//while ((r.minY < r.maxY) && (lineDy (r.maxY) == 0)) r.maxY--; 
				while ((r.minY < r.maxY) && (lineDx (r.minY) == 0)) r.minY++; 
				while ((r.minY < r.maxY) && (lineDx (r.maxY) == 0)) r.maxY--; 
		
				
				// split text regions into two regions (check for two text lines)
				int32 split = 0;
				for (int32 y=r.minY; y<=r.maxY; y++) {
					if (lineDx(y) == 0) split = y;
				} // for
				if (split != 0) {

					//add textbox 1
					TextRegionRect r0 = data.box[i];
					r0.minY = split;
					data.box.push_back(r0);
					
					//add textbox 2
					r0 = data.box[i];
					r0.maxY = split;
					data.box.push_back(r0);

					//delete old textbox 
					data.box[i].delData = true;
				} else {
					
					// find exact position of the border of the text line (based on dx)
					int32 center1 = (r.minY+r.minY+r.maxY)/3;
					int32 center2 = (r.minY+r.maxY+r.maxY)/3;
					int32 y0 = r.minY;
					for (int32 y=r.minY; y<center2; y++) {
						if (lineDx (y0+1)-lineDx(y0) < lineDx (y+1)-lineDx(y)) {
							y0 = y;
						} // if
					} // for
					int32 y1 = r.maxY-2;
					for (int32 y=r.maxY-2; y>center1; y--) {
						if (lineDx (y1)-lineDx(y1+1) < lineDx (y)-lineDx(y+1)) {
							y1 = y;
						} // if
					} // for

					// store position
					data.box[i].minY = y0;
					data.box[i].maxY = y1;


					// verify position with dy
					center1 = data.box[i].minY-5; if (center1<0) center1=0;
					for (int32 j=center1; j<=data.box[i].minY; j++) {
						if (lineDy(y0) < lineDy(j)) y0=j;
					} // for
					center2 = data.box[i].maxY+5; if (center2>=size[DIM_Y]) center2=size[DIM_Y]-1;
					for (int32 j=data.box[i].maxY; j<=center2; j++) {
						if (lineDy(y1) < lineDy(j)) y1=j;
					} // for



					// store position
					data.box[i].minY = y0;
					data.box[i].maxY = y1;
					data.box[i].analysed = true;

				} // else
			} // if
		} // for

		// if rect is split, analyse again
		rectAnalysed = true;
		for (int32 i=0; i<data.box.size(); i++) {
			if ((!data.box[i].analysed) && (!data.box[i].delData)) { 
				rectAnalysed = false;
			} // if
		} // for

	} // while
	debugStoreTextRegions (input, data, "regions02.png");


	// delete very small text boxes
	for (int32 i=0; i<data.box.size(); i++) {
		if (!data.box[i].delData) {
			int32 v = data.box[i].maxX - data.box[i].minX;
			if (v<thTextWidth) {
				data.box[i].delData = true;
			} // if
			v = data.box[i].maxY - data.box[i].minY;
			if (v<thTextHeight) {
				data.box[i].delData = true;
			} // if 
		} // if
	} // for
	debugStoreTextRegions (input, data, "regions03.png");


	// detect exact width of text areas
	Image<uint8> dxMod (dxSize);
	dxMod.clear(0);
	for (int32 i=0; i<data.box.size(); i++) {
		if (!data.box[i].delData) {
			int32 width = (data.box[i].maxY - data.box[i].minY)/10 + 1;

			int32 y0 = data.box[i].minY - width; if (y0<0) y0=0;
			int32 y1 = data.box[i].minY + width; if (y1>=size[DIM_Y]) y1=size[DIM_Y]-1;
			for (int32 y=y0; y<=y1; y++) {
				for (int32 x=0; x<size[DIM_X]; x++) {
					dxMod (x,y) = dy (x,y);
				} // for
			} // for

			y0 = data.box[i].maxY - width; if (y0<0) y0=0;
			y1 = data.box[i].maxY + width; if (y1>=size[DIM_Y]) y1=size[DIM_Y]-1;
			for (int32 y=y0; y<=y1; y++) {
				for (int32 x=0; x<size[DIM_X]; x++) {
					dxMod (x,y) = dy (x,y);
				} // for
			} // for

			y0 = data.box[i].minY; 
			y1 = data.box[i].maxY; 
			for (int32 y=y0; y<=y1; y++) {
				for (int32 x=0; x<size[DIM_X]; x++) {
					if (dxMod (x,y) < dx (x,y)) 
						dxMod (x,y) = dx (x,y);
				} // for
			} // for

		} // if
	} // for
	debugStoreTextRegions (input, data, "regions04.png");

	// summarize text column for each text line
	SignalSize cSize (size[DIM_X]);
	Signal<int32> column (cSize);
	Signal<int32> column2 (cSize);
	for (int32 i=0; i<data.box.size(); i++) {
		if (!data.box[i].delData) {

			// initilaize column
			column.clear(0); 
			int32 y0 = data.box[i].minY; 
			int32 y1 = data.box[i].maxY; 
			for (int32 y=y0; y<=y1; y++) {
				for (int32 x=0; x<size[DIM_X]; x++) {
					if (dxMod (x,y)>0) 
						column (x)++;
				} // for
			} // for


			// locate characters
			int32 maxV = 0;
			for (int32 x=0; x<size[DIM_X]; x++) {
				if (maxV < column(x)) maxV=column(x);
			} // for
			maxV = (int32) ((0.9*maxV)-1);
			if (maxV<3) maxV=3;
			int32 width = y1-y0+1;

			column2.clear(0);
			for (int32 x=0; x<size[DIM_X]; x++) {
				if (column(x)>maxV) {
					int32 x0 = x-width; if (x0<0) x0=0;
					int32 x1 = x+width; if (x1>=size[DIM_X]) x1=size[DIM_X]-1;
					for (int32 y=x0; y<=x1; y++) {
						column2(y) = column(x);
					} // for

				} // if
			} // for
			
			// set new border (left)
			int32 x0 = data.box[i].minX;
			while ((x0>0) && (column2(x0) >= maxV)) x0--;
			while ((x0<size[DIM_X]) && (column2(x0) < maxV)) x0++;
			while ((x0<size[DIM_X]) && (column(x0) < maxV)) x0++;
			x0--; if (x0<0) x0=0;
			data.box[i].minX = x0;

			// set new border (left)
			x0 = data.box[i].maxX;
			while ((x0<size[DIM_X]) && (column2(x0) >= maxV)) x0++;
			while ((x0>0) && (column2(x0) < maxV)) x0--;
			while ((x0>0) && (column(x0) < maxV)) x0--;
			x0++; if (x0>=size[DIM_X]) x0=size[DIM_X]-1;
			data.box[i].maxX = x0;

		} // if
	} // for
	debugStoreTextRegions (input, data, "regions05.png");




	// update width (somtime some characters are lost!)
	// use histogram
	SignalSize histSize (256);
	Signal<ieee64> hist1 (histSize); // text
	Signal<ieee64> hist2 (histSize); // non text



	// approximate text color
	for (int32 i=0; i<data.box.size(); i++) {
		if (!data.box[i].delData) {

			// get bounding box
			int32 maxX = data.box[i].maxX;
			int32 minX = data.box[i].minX;

			int32 maxY = data.box[i].maxY;
			int32 minY = data.box[i].minY;
			int32 height = maxY-minY+1;

			int32 minY2 = minY - height/2; 
			if (minY2<0) minY2=0;

			int32 maxY2 = minY2 + 2* height; 
			if (maxY2 > size[DIM_Y]) {
				maxY2 = size[DIM_Y];
			} // if

			minY-=minY2;
			maxY-=minY2;

			// crop image
			Image<uint8> tmp;
			ImagePosition p0(0,minY2);
			ImagePosition p1(size[DIM_X],maxY2);
			CropFilter<uint8> crop(p0,p1);
			crop.process(input,tmp);

			// convert color space YUV8
			VideoCodec srcMode = CODEC_RGB;
			VideoCodec destMode= CODEC_YUV8;
			ConvertColorSpace::process(srcMode, destMode, tmp);
			ImageSize tmpSize = tmp.getSize();

			// get histograms
			hist1.clear(0);
			hist2.clear(0);
			ieee64 height1=0;
			ieee64 height2=0;
			for (int32 y=0; y<tmpSize[DIM_Y]; y++) {  
				for (int32 x=0; x<tmpSize[DIM_X]; x++) {  
					if ((y>minY) && (y<maxY)) {
						hist1 (tmp(x,y))++;
						height1++;
					} else {
						hist2 (tmp(x,y))++;
						height2++;
					} // else
				} // for
			} // for
			for (int32 u=0; u<256; u++) {
				hist1(u) /= height1;
				hist2(u) /= height2;
			} // for

			int32 p=0;
			ieee64 maxDist = -10e10;
			for (int32 u=0; u<256; u++) {
				ieee64 d = hist1(u) -hist2(u);
				if (maxDist < d) {
					maxDist = d;
					p=u;
				}
			} // for

			// count the number of text pixels
			SignalSize nSize(size[DIM_X]);
			Signal<int32> n(nSize);
			n.clear(0);
			for (int32 y=minY; y<maxY; y++) {  
				for (int32 x=0; x<tmpSize[DIM_X]; x++) {  
					if (tmp(x,y) == p) n(x)++;
				} // for
			} // for
			ieee64 avg=0;
			for (int32 x=0; x<tmpSize[DIM_X]; x++) {  
				avg+= n(x);
			} // for
			avg/=tmpSize[DIM_X];

			bool found = true;
			while (found) {
				found=false;
				int32 x0 = minX;
				int32 x1 = x0-30; if (x1<0) x1=0;
				for (int32 i=x0; i>x1; i--) {
					if (n(i)>avg) {
						found=true;
						minX=i-1;
						break;
					} // if
				} // for
			} // while
			if (minX<1) minX=1;
			
			found = true;
			while (found) {
				found=false;
				int32 x0 = maxX;
				int32 x1 = x0+30; if (x1>size[DIM_X]-1) x1=size[DIM_X]-1;
				for (int32 i=x0; i<x1; i++) {
					if (n(i)>avg) {
						found=true;
						maxX=i+1;
						break;
					} // if
				} // for
			} // while
			if (maxX>size[DIM_X]-2) maxX = size[DIM_X]-2;

			// update minX, maxX
			data.box[i].maxX = maxX;
			data.box[i].minX = minX;

		} // if
	} // for
	debugStoreTextRegions (input, data, "regions06.png");
	

	// store and sort text regions
	bool regionFound=true;
	result.size = 0;
	while (regionFound) {

		regionFound = false;
		int32 minR = -1;
		for (int32 i=0; i<data.box.size(); i++) {
			if (!data.box[i].delData) {

				if (minR==-1) minR = i;
				if (data.box[minR].minY > data.box[i].minY) minR = i;
			} // if
		} // for
		if (minR != -1) {
			regionFound = true;
			result.size++;
			result.box.push_back(data.box[minR]);
			data.box[minR].delData = true;
		} // if
	} // while
	debugStoreTextRegions (input, data, "regions07.png");
	debugStoreTextRegions (input, result, "regions08.png");

	return;
} // process

