/*
    VideoStatistic.cpp     get statistical values of videos
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <VideoStatistic.h>


std::ostream& operator<< (ostream& os, const VideoStatisticData &data)
{
	os << "version        : " << data.version         << endl;
	os << "filename       : " << data.filename        << endl;
	os << "frames         : " << data.frames << endl;
	
	os << "avg: " << data.avgVideoData << endl;
	os << "min: " << data.minVideoData << endl;
	os << "max: " << data.maxVideoData << endl;

	os << "color frames    : " << data.numberOfColorFrames << endl;
	os << "near gray frames: " << data.numberOfNearGrayFrames<< endl;
	os << "gray frames     : " << data.numberOfGrayFrames << endl;

	if (data.color == TYPE_GRAYSCALE)
		os << "video mode      : GRAY" << endl;
	if (data.color == TYPE_NEAR_GRAYSCALE)
		os << "video mode      : NEAR GRAY" << endl;
	if (data.color == TYPE_COLOR)
		os << "video mode      : COLOR" << endl;

	return os;
} // operator<<

OTagStream& operator<< (OTagStream& os, const VideoStatisticData& data)
{
	
    TagFormat::writeStartTag (os, "statistics");
	os << endl;
	
    TagFormat::writeNodeValue(os, "version", data.version, 1);
    TagFormat::writeNodeValue(os, "filename", data.filename, 1);
    TagFormat::writeNodeValue(os, "frames", data.frames, 1);

    TagFormat::writeNodeValue(os, "colorFrames",    data.numberOfColorFrames, 1);
    TagFormat::writeNodeValue(os, "nearGrayFrames", data.numberOfNearGrayFrames, 1);
    TagFormat::writeNodeValue(os, "grayFrames",     data.numberOfGrayFrames, 1);

	string s = "UNKNOWN";
	if (data.color == TYPE_GRAYSCALE)
		s = "GRAY";
	if (data.color == TYPE_NEAR_GRAYSCALE)
		s = "NEAR_GRAY";
	if (data.color == TYPE_COLOR)
		s = "COLOR";
    TagFormat::writeNodeValue(os, "videoMode",s, 1);

	// comment
	os << endl;
	os << "  <comment>       average(YUV-plane)   sigma(YUV-plane)   contrast(max,hist,scale)   color </comment>" << endl;

	// store: 
	//   average (for each of the 3 BANDS), 
	//   sigma   (for each of the 3 BANDS), 
	//   contrast (contrastHistogram and contrastScale) 

	
    TagFormat::writeStartTag(os, "avgVideoData", 1);	
	int32 v=data.avgVideoData.avg[0];
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.avgVideoData.avg[1];
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.avgVideoData.avg[2];
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;	

	v=data.avgVideoData.sigma[0];
	if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; 
	os << "   " << v;
	v=data.avgVideoData.sigma[1];
	if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; 
	os << "   " << v;
	v=data.avgVideoData.sigma[2];
	if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; 
	os << "   " << v << "   ";	

	v=data.avgVideoData.contrastMaxMin;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.avgVideoData.contrastHistogram;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.avgVideoData.contrastScale;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v << "             ";
	v=data.avgVideoData.avgEdgeStrength;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v << "             ";
    TagFormat::writeEndTag(os, "avgVideoData", 1);
	
	
    TagFormat::writeStartTag(os, "minVideoData", 1);	
	v=data.minVideoData.avg[0];
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.minVideoData.avg[1];
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.minVideoData.avg[2];
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;	
	v=data.minVideoData.sigma[0];
	if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; 
	os << "   " << v;
	v=data.minVideoData.sigma[1];
	if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; 
	os << "   " << v;
	v=data.minVideoData.sigma[2];
	if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; 
	os << "   " << v << "   ";	
	v=data.minVideoData.contrastMaxMin;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.minVideoData.contrastHistogram;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.minVideoData.contrastScale;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v << "             ";
	v=data.minVideoData.avgEdgeStrength;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v << "             ";
    TagFormat::writeEndTag(os, "minVideoData", 1);
	
	
    TagFormat::writeStartTag(os, "maxVideoData", 1);	
	v=data.maxVideoData.avg[0];
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.maxVideoData.avg[1];
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.maxVideoData.avg[2];
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;	
	v=data.maxVideoData.sigma[0];
	if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; 
	os << "   " << v;
	v=data.maxVideoData.sigma[1];
	if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; 
	os << "   " << v;
	v=data.maxVideoData.sigma[2];
	if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; 
	os << "   " << v << "   ";	
	v=data.maxVideoData.contrastMaxMin;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.maxVideoData.contrastHistogram;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v;
	v=data.maxVideoData.contrastScale;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v << "             ";
	v=data.maxVideoData.avgEdgeStrength;
	if (v<10) os << " ";  if (v<100) os << " "; 
	os << "   " << v << "             ";
    TagFormat::writeEndTag(os, "maxVideoData", 1);
	os << endl;
	

    TagFormat::writeStartTag(os, "data", 1);
	os << endl;
	v=0;
	for (int32 i=0; i<data.frames; i++) {
		if (i<10) os << " ";  if (i<100) os << " "; if (i<1000)  os << " "; if (i<10000)  os << " ";
		os << i;
		
		v=data.imgData[i].avg[0];
		if (v<10) os << " ";  if (v<100) os << " "; 
		os << "   " << v;
		v=data.imgData[i].avg[1];
		if (v<10) os << " ";  if (v<100) os << " "; 
		os << "   " << v;
		v=data.imgData[i].avg[2];
		if (v<10) os << " ";  if (v<100) os << " "; 
		os << "   " << v;
		
		v=data.imgData[i].sigma[0];
		if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; if (v<10000)  os << " ";
		os << "   " << v;
		v=data.imgData[i].sigma[1];
		if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; if (v<10000)  os << " ";
		os << "   " << v;
		v=data.imgData[i].sigma[2];
		if (v<10) os << " ";  if (v<100) os << " "; if (v<1000)  os << " "; if (v<10000)  os << " ";
		os << "   " << v;
		
		v=data.imgData[i].contrastMaxMin;
		if (v<10) os << " ";  if (v<100) os << " "; 
		os << "   " << v;
		v=data.imgData[i].contrastHistogram;
		if (v<10) os << " ";  if (v<100) os << " "; 
		os << "   " << v;
		v=data.imgData[i].contrastScale;
		if (v<10) os << " ";  if (v<100) os << " "; 
		os << "   " << v;
		v=data.imgData[i].avgEdgeStrength;
		if (v<10) os << " ";  if (v<100) os << " "; 
		os << "   " << v;

		string s = "     UNKNOWN         ";
		if (data.imgData[i].color == TYPE_COLOR)          s = "     COLOR          ";
		if (data.imgData[i].color == TYPE_GRAYSCALE)      s = "     GRAYSCALE      ";
		if (data.imgData[i].color == TYPE_NEAR_GRAYSCALE) s = "     NEAR_GRAYSCALE ";
		os << s;

		os << endl;
	} // for
    TagFormat::writeEndTag(os, "data", 1);
	os << endl;
	
    TagFormat::writeEndTag(os, "statistics");
	os << endl;
	
    return os;
} // operator<<

ITagStream& operator>> (ITagStream& is, VideoStatisticData& data)
{
    bool failFlag = false;
    TagFormat::readStartTag(is, "statistics");
	
    TagFormat::readNodeValue(is, "version", data.version);
    TagFormat::readNodeValue(is, "filename", data.filename);
    TagFormat::readNodeValue(is, "frames", data.frames);


    TagFormat::readNodeValue(is, "colorFrames", data.numberOfColorFrames);
    TagFormat::readNodeValue(is, "nearGrayFrames", data.numberOfNearGrayFrames);
    TagFormat::readNodeValue(is, "grayFrames", data.numberOfGrayFrames);

	string s = "";
    TagFormat::readNodeValue(is, "videoMode", s);
	data.color = TYPE_UNKNOWN;
	if (s == "GRAY")
		data.color = TYPE_GRAYSCALE;
	if (s == "NEAR_GRAY")
		data.color = TYPE_NEAR_GRAYSCALE;
	if (s == "COLOR")
		data.color = TYPE_COLOR;


	SignalSize size (data.frames);
	data.imgData.resize(data.frames);

	// read comment
	string tmp;
    TagFormat::readNodeValue(is, "comment", tmp);


    TagFormat::readStartTag(is, "avgVideoData");
	int32 i;
	is >> i; data.avgVideoData.avg[0] = i;
	is >> i; data.avgVideoData.avg[1] = i;
	is >> i; data.avgVideoData.avg[2] = i;
	is >> i; data.avgVideoData.sigma[0] = i;
	is >> i; data.avgVideoData.sigma[1] = i;
	is >> i; data.avgVideoData.sigma[2] = i;

	is >> i; data.avgVideoData.contrastMaxMin = i;
	is >> i; data.avgVideoData.contrastHistogram = i;
	is >> i; data.avgVideoData.contrastScale     = i;
	is >> i; data.avgVideoData.avgEdgeStrength     = i;
    TagFormat::readEndTag(is, "avgVideoData");
	
    TagFormat::readStartTag(is, "minVideoData");
	is >> i; data.minVideoData.avg[0] = i;
	is >> i; data.minVideoData.avg[1] = i;
	is >> i; data.minVideoData.avg[2] = i;
	is >> i; data.minVideoData.sigma[0] = i;
	is >> i; data.minVideoData.sigma[1] = i;
	is >> i; data.minVideoData.sigma[2] = i;

	is >> i; data.minVideoData.contrastMaxMin = i;
	is >> i; data.minVideoData.contrastHistogram = i;
	is >> i; data.minVideoData.contrastScale = i;
	is >> i; data.minVideoData.avgEdgeStrength = i;
    TagFormat::readEndTag(is, "minVideoData");
	
    TagFormat::readStartTag(is, "maxVideoData");
	is >> i; data.maxVideoData.avg[0] = i;
	is >> i; data.maxVideoData.avg[1] = i;
	is >> i; data.maxVideoData.avg[2] = i;
	is >> i; data.maxVideoData.sigma[0] = i;
	is >> i; data.maxVideoData.sigma[1] = i;
	is >> i; data.maxVideoData.sigma[2] = i;

	is >> i; data.maxVideoData.contrastMaxMin = i;
	is >> i; data.maxVideoData.contrastHistogram = i;
	is >> i; data.maxVideoData.contrastScale = i;
	is >> i; data.maxVideoData.avgEdgeStrength = i;
    TagFormat::readEndTag(is, "maxVideoData");
	
    TagFormat::readStartTag(is, "data");
	if (is.rdstate() & ios::failbit) {
		failFlag = true;
	} // if
	
	// read values
	for (int i=0; i<size[DIM_X]; i++) {
		int32 frame = 0;
		is >> frame;

		int32 v;
		is >> v; data.imgData[i].avg[0] = v;
		is >> v; data.imgData[i].avg[1] = v;
		is >> v; data.imgData[i].avg[2] = v;
		
		is >> v; data.imgData[i].sigma[0] = v;
		is >> v; data.imgData[i].sigma[1] = v;
		is >> v; data.imgData[i].sigma[2] = v;
		
		is >> v; data.imgData[i].contrastMaxMin = v;
		is >> v; data.imgData[i].contrastHistogram = v;
		is >> v; data.imgData[i].contrastScale = v;
		is >> v; data.imgData[i].avgEdgeStrength = v;

		string s = "";
		data.imgData[i].color = TYPE_UNKNOWN;
		is >> s;
		if (s == "COLOR")          data.imgData[i].color = TYPE_COLOR;
		if (s == "GRAYSCALE")      data.imgData[i].color = TYPE_GRAYSCALE;
		if (s == "NEAR_GRAYSCALE") data.imgData[i].color = TYPE_NEAR_GRAYSCALE;
	} // for
	
    TagFormat::readEndTag(is, "data");
    TagFormat::readEndTag(is, "statistics");
	
	if (is.rdstate() & ios::failbit) {
		failFlag = true;
	} // if
    if (failFlag) {
		throw MocaException ("VideoStatistic: Illegal index file!");
	} // if
	
    return is;
} // operator>> 




void 
VideoStatistic::process(VideoStatisticData &data) 
{
	// read data
	ReadMocaData<VideoStatisticData> reader (pathFilename, VIDEO_STATISTIC_FILENAME);
	bool dataValid = reader.process (VIDEO_STATISTIC_VERSION, data); 
	
	if (!dataValid) {
		
		// create new index
		cerr << "\rVideoStatistic : Creating index file ...         " << endl;
		readValues(data);
		
		// save index file
		WriteMocaData<VideoStatisticData> writer (pathFilename, VIDEO_STATISTIC_FILENAME);
		writer.process(data);
		cerr << "\rVideoStatistic : Creating index file ... done         " << endl;
	} // if

	return;
} // process


// do not calculate values, but use already calculated data of frames,
// and store them in data
void 
VideoStatistic::process (list<ImageStatisticData> &imgStatData, VideoStatisticData &data)
{
	// read data
	ReadMocaData<VideoStatisticData> reader (pathFilename, VIDEO_STATISTIC_FILENAME);
	bool dataValid = reader.process (VIDEO_STATISTIC_VERSION, data); 
	
	// if data not valid, store imgStatData
	if (!dataValid) {
		
		// set values
		cerr << "\rVideoStatistic : Creating index file ...         " << endl;
		readValues(imgStatData, data);
		
		// save index file
		WriteMocaData<VideoStatisticData> writer (pathFilename, VIDEO_STATISTIC_FILENAME);
		writer.process(data);
		cerr << "\rVideoStatistic : Creating index file ... done         " << endl;
	} // if
	
	return;
} // process




void 
VideoStatistic::readValues (VideoStatisticData &data) 
{
	// read information
	VideoCodec codec = CODEC_YUV444;
	VideoDecoder reader (pathFilename, 3, -1, codec);
	VideoSize size = reader.getSize();

	// get path of filename, do not create new directory
	CreateMocaPath mocaPath (pathFilename, false);
	data.filename = mocaPath.file;
	
	data.frames   = size[DIM_Z];
	data.version  = VIDEO_STATISTIC_VERSION;
	data.imgData.resize(data.frames);
	
	int32 avg0 = 0;
	int32 avg1 = 0;
	int32 avg2 = 0;
	int32 var0 = 0;
	int32 var1 = 0;
	int32 var2 = 0;
	int32 conMaxMin = 0;
	int32 conHist   = 0;
	int32 conScale  = 0;
	int32 avgEdge   = 0;
	for (int k=0; k<3; k++) {
		data.minVideoData.avg[k]      = 255;
		data.minVideoData.sigma[k] = 100000000;
		data.maxVideoData.avg[k]      = 0;
		data.maxVideoData.sigma[k] = -1;
	} // for

	data.minVideoData.contrastMaxMin = 255;
	data.minVideoData.contrastHistogram = 255;
	data.minVideoData.contrastScale     = 255;
	data.minVideoData.avgEdgeStrength     = 255;

	data.maxVideoData.contrastMaxMin = 0;
	data.maxVideoData.contrastHistogram = 0;
	data.maxVideoData.contrastScale = 0;
	data.maxVideoData.avgEdgeStrength = 0;
	
	Image<uint8> img;
	ImageStatistic imgStat;
	for (int i=0; i<size[DIM_Z]; i++) {
		img = reader.getImage(i);
		cerr << "\rframe: " << i << "    " << ends;

		imgStat.process(img, data.imgData[i]);
		
		avg0 += data.imgData[i].avg[0];
		avg1 += data.imgData[i].avg[1];
		avg2 += data.imgData[i].avg[2];
		
		var0 += data.imgData[i].sigma[0];
		var1 += data.imgData[i].sigma[1];
		var2 += data.imgData[i].sigma[2];
		
		conMaxMin += data.imgData[i].contrastMaxMin;
		conHist   += data.imgData[i].contrastHistogram;
		conScale  += data.imgData[i].contrastScale;
		avgEdge   += data.imgData[i].avgEdgeStrength;
		
		for (int k=0; k<3; k++) {
			if (data.minVideoData.avg[k] > data.imgData[i].avg[k]) {
				data.minVideoData.avg[k] = data.imgData[i].avg[k];
			} // if
			if (data.minVideoData.sigma[k] > data.imgData[i].sigma[k]) {
				data.minVideoData.sigma[k] = data.imgData[i].sigma[k];
			} // if
			
			if (data.maxVideoData.avg[k] < data.imgData[i].avg[k]) {
				data.maxVideoData.avg[k] = data.imgData[i].avg[k];
			} // if
			if (data.maxVideoData.sigma[k] < data.imgData[i].sigma[k]) {
				data.maxVideoData.sigma[k] = data.imgData[i].sigma[k];
			} // if
		} // for

		if (data.minVideoData.contrastMaxMin > data.imgData[i].contrastMaxMin) {
			data.minVideoData.contrastMaxMin = data.imgData[i].contrastMaxMin;
		} // if
		if (data.minVideoData.contrastHistogram > data.imgData[i].contrastHistogram) {
			data.minVideoData.contrastHistogram = data.imgData[i].contrastHistogram;
		} // if
		if (data.minVideoData.contrastScale > data.imgData[i].contrastScale) {
			data.minVideoData.contrastScale = data.imgData[i].contrastScale;
		} // if
		if (data.minVideoData.avgEdgeStrength > data.imgData[i].avgEdgeStrength) {
			data.minVideoData.avgEdgeStrength = data.imgData[i].avgEdgeStrength;
		} // if

		if (data.maxVideoData.contrastMaxMin < data.imgData[i].contrastMaxMin) {
			data.maxVideoData.contrastMaxMin = data.imgData[i].contrastMaxMin;
		} // if
		if (data.maxVideoData.contrastHistogram < data.imgData[i].contrastHistogram) {
			data.maxVideoData.contrastHistogram = data.imgData[i].contrastHistogram;
		} // if
		if (data.maxVideoData.contrastScale < data.imgData[i].contrastScale) {
			data.maxVideoData.contrastScale = data.imgData[i].contrastScale;
		} // if
		if (data.maxVideoData.avgEdgeStrength < data.imgData[i].avgEdgeStrength) {
			data.maxVideoData.avgEdgeStrength = data.imgData[i].avgEdgeStrength;
		} // if
	} // for

	avg0 /= size[DIM_Z];
	avg1 /= size[DIM_Z];
	avg2 /= size[DIM_Z];
	var0 /= size[DIM_Z];
	var1 /= size[DIM_Z];	
	var2 /= size[DIM_Z];
	conMaxMin /= size[DIM_Z];
	conHist   /= size[DIM_Z];
	conScale  /= size[DIM_Z];
	avgEdge   /= size[DIM_Z];
	
	data.avgVideoData.avg[0] = (uint8)avg0;
	data.avgVideoData.avg[1] = (uint8)avg1;
	data.avgVideoData.avg[2] = (uint8)avg2;
	data.avgVideoData.sigma[0] = var0;
	data.avgVideoData.sigma[1] = var1;
	data.avgVideoData.sigma[2] = var2;

	data.avgVideoData.contrastMaxMin    = (uint8)conMaxMin;
	data.avgVideoData.contrastHistogram = (uint8)conHist;
	data.avgVideoData.contrastScale     = (uint8)conScale;
	data.avgVideoData.avgEdgeStrength   = (uint8)avgEdge;
	
	//cerr << endl;
	// analyse type of video (color, grayscale, ...)
	data.numberOfColorFrames    = 0;
	data.numberOfNearGrayFrames = 0;
	data.numberOfGrayFrames     = 0;
	data.color = TYPE_UNKNOWN;

	for (int i=0; i<data.frames; i++) {
		GrayscaleType color = data.imgData[i].color;
		if (color == TYPE_GRAYSCALE)      data.numberOfGrayFrames++;
		if (color == TYPE_NEAR_GRAYSCALE) data.numberOfNearGrayFrames++;
		if (color == TYPE_COLOR)          data.numberOfColorFrames++;
	} // for

	int32 framesColor    = (int32)(1000.0 * data.numberOfColorFrames    / data.frames);
	int32 framesNearGray = (int32)(1000.0 * data.numberOfNearGrayFrames / data.frames);
	//int32 framesGray     = (int32)(1000.0 * data.numberOfColorFrames    / data.frames);

	// set video color
	data.color = TYPE_GRAYSCALE;

	// nearGray, if more than 2.0 percent nearGray frames
	if (framesNearGray > 20) data.color = TYPE_NEAR_GRAYSCALE; 
	// color, if more than 0.5 percent color frames
	if (framesColor > 5) data.color = TYPE_COLOR; 
	// color, if more than 0.1 percent color frames and
	//           more than 1.5 percent nearGray frames 
	if ((framesColor > 1) && (framesNearGray > 15)) data.color = TYPE_COLOR;


	return;
} // init



void 
VideoStatistic::readValues (list<ImageStatisticData> &imgStatData, VideoStatisticData &data) 
{
	// get path of filename, do not create new directory
	CreateMocaPath mocaPath (pathFilename, false);
	data.filename = mocaPath.file;
	
	data.frames   = imgStatData.size();
	data.version  = VIDEO_STATISTIC_VERSION;
	data.imgData.resize(data.frames);
	
	int32 avg0 = 0;
	int32 avg1 = 0;
	int32 avg2 = 0;
	int32 var0 = 0;
	int32 var1 = 0;
	int32 var2 = 0;
	int32 conMaxMin = 0;
	int32 conHist   = 0;
	int32 conScale  = 0;
	int32 avgEdge   = 0;
	for (int k=0; k<3; k++) {
		data.minVideoData.avg[k]      = 255;
		data.minVideoData.sigma[k] = 100000000;
		data.maxVideoData.avg[k]      = 0;
		data.maxVideoData.sigma[k] = -1;
	} // for

	data.minVideoData.contrastMaxMin = 255;
	data.minVideoData.contrastHistogram = 255;
	data.minVideoData.contrastScale     = 255;
	data.minVideoData.avgEdgeStrength   = 255;

	data.maxVideoData.contrastMaxMin = 0;
	data.maxVideoData.contrastHistogram = 0;
	data.maxVideoData.contrastScale = 0;
	data.maxVideoData.avgEdgeStrength = 0;
	
	list<ImageStatisticData>::iterator it = imgStatData.begin();
	ImageStatisticData item;
	for (int i=0; i<data.frames; i++) {

		item = (*it);
		data.imgData[i] = item;
		if (it!=imgStatData.end()) it++;
		cerr << "\rframe: " << i << "    " << ends;
		
		avg0 += item.avg[0];
		avg1 += item.avg[1];
		avg2 += item.avg[2];
		
		var0 += item.sigma[0];
		var1 += item.sigma[1];
		var2 += item.sigma[2];
		
		conMaxMin += item.contrastMaxMin;
		conHist   += item.contrastHistogram;
		conScale  += item.contrastScale;
		avgEdge  += item.avgEdgeStrength;
		
		for (int k=0; k<3; k++) {
			if (data.minVideoData.avg[k] > item.avg[k]) {
				data.minVideoData.avg[k] = item.avg[k];
			} // if
			if (data.minVideoData.sigma[k] > item.sigma[k]) {
				data.minVideoData.sigma[k] = item.sigma[k];
			} // if
			
			if (data.maxVideoData.avg[k] < item.avg[k]) {
				data.maxVideoData.avg[k] = item.avg[k];
			} // if
			if (data.maxVideoData.sigma[k] < item.sigma[k]) {
				data.maxVideoData.sigma[k] = item.sigma[k];
			} // if
		} // for

		if (data.minVideoData.contrastMaxMin >item.contrastMaxMin) {
			data.minVideoData.contrastMaxMin = item.contrastMaxMin;
		} // if
		if (data.minVideoData.contrastHistogram > item.contrastHistogram) {
			data.minVideoData.contrastHistogram = item.contrastHistogram;
		} // if
		if (data.minVideoData.contrastScale > item.contrastScale) {
			data.minVideoData.contrastScale = item.contrastScale;
		} // if
		if (data.minVideoData.avgEdgeStrength > item.avgEdgeStrength) {
			data.minVideoData.avgEdgeStrength = item.avgEdgeStrength;
		} // if

		if (data.maxVideoData.contrastMaxMin < item.contrastMaxMin) {
			data.maxVideoData.contrastMaxMin = item.contrastMaxMin;
		} // if
		if (data.maxVideoData.contrastHistogram < item.contrastHistogram) {
			data.maxVideoData.contrastHistogram = item.contrastHistogram;
		} // if
		if (data.maxVideoData.contrastScale < item.contrastScale) {
			data.maxVideoData.contrastScale = item.contrastScale;
		} // if
		if (data.maxVideoData.avgEdgeStrength < item.avgEdgeStrength) {
			data.maxVideoData.avgEdgeStrength = item.avgEdgeStrength;
		} // if
	} // for

	avg0 /= data.frames;
	avg1 /= data.frames;
	avg2 /= data.frames;
	var0 /= data.frames;
	var1 /= data.frames;
	var2 /= data.frames;
	conMaxMin /= data.frames;
	conHist   /= data.frames;
	conScale  /= data.frames;
	avgEdge   /= data.frames;
	
	data.avgVideoData.avg[0] = (uint8)avg0;
	data.avgVideoData.avg[1] = (uint8)avg1;
	data.avgVideoData.avg[2] = (uint8)avg2;
	data.avgVideoData.sigma[0] = var0;
	data.avgVideoData.sigma[1] = var1;
	data.avgVideoData.sigma[2] = var2;

	data.avgVideoData.contrastMaxMin    = (uint8)conMaxMin;
	data.avgVideoData.contrastHistogram = (uint8)conHist;
	data.avgVideoData.contrastScale     = (uint8)conScale;
	data.avgVideoData.avgEdgeStrength     = (uint8)avgEdge;

	
	// analyse type of video (color, grayscale, ...)
	data.numberOfColorFrames    = 0;
	data.numberOfNearGrayFrames = 0;
	data.numberOfGrayFrames     = 0;
	data.color = TYPE_UNKNOWN;

	for (int i=0; i<data.frames; i++) {
		GrayscaleType color = data.imgData[i].color;
		if (color == TYPE_GRAYSCALE)      data.numberOfGrayFrames++;
		if (color == TYPE_NEAR_GRAYSCALE) data.numberOfNearGrayFrames++;
		if (color == TYPE_COLOR)          data.numberOfColorFrames++;
	} // for

	int32 framesColor    = (int32)(1000.0 * data.numberOfColorFrames    / data.frames);
	int32 framesNearGray = (int32)(1000.0 * data.numberOfNearGrayFrames / data.frames);
	//int32 framesGray     = (int32)(1000.0 * data.numberOfColorFrames    / data.frames);

	// set video color
	data.color = TYPE_GRAYSCALE;

	// nearGray, if more than 2.0 percent nearGray frames
	if (framesNearGray > 20) data.color = TYPE_NEAR_GRAYSCALE; 
	// color, if more than 0.5 percent color frames
	if (framesColor > 5) data.color = TYPE_COLOR; 
	// color, if more than 0.1 percent color frames and
	//           more than 1.5 percent nearGray frames 
	if ((framesColor > 1) && (framesNearGray > 15)) data.color = TYPE_COLOR;

	return;
} // init



