/*
   This file is part of the MoCA video library.
   Name: AndFilter.h
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: AndFilter.h,v 1.1 2004/02/17 11:53:52 kopf Exp $

*/

#ifndef AND_FILTER_H
#define AND_FILTER_H

// MoCA includes
#include <ImageFilterBase.h>


/*!

  \brief a simple and filter

  This filter is an "and" filter. It takes two images as argument and
  results in a third image. The third image contains only pixels at
  positions where both the first and second image contain pixel.

  More precisely, it writes the minimum pixel value of each pixel from
  the first and second image into the third image.
  
 */
template<class T>
class AndFilter 
{
public:
    //! the default constructor
    AndFilter(){}

    //! the destructor
    ~AndFilter(){};

    //! the filter
    void process(const Image<T> &src, Image<T> &srcDest);

    //! the filter
    void process(const Image<T> &one, const Image<T> &two, Image<T> &diff);

}; // AndFilter

#include <AndFilter.cpt>

#endif
