/*
   This file is part of the MoCA video library.
   Name: BinaryContourFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef BINARY_CONTOUR_EXTRACTION_H
#define BINARY_CONTOUR_EXTRACTION_H

#include <ImageFilterBase.h>

/*!
  \brief Extract the contour of a binary image

  This filter expects a binary image. The two colours can be
  parametrized in the contructor. The main work is done in
  filter(). It copies only the outer pixels of any region to the
  destination image.

*/
template<class T>
class BinaryContourFilter : public ImageFilterBase<T>
{
public:
    /*!
      The contructor. Provide the two colours of the image.
      \param fGroundColor one of the two colors, interpreted as the foreground color
      \param bGroundColor the other color, interpreted as the background color
    */
    BinaryContourFilter(T fGroundColor, T bGroundColor)
        : fGroundColor(fGroundColor), bGroundColor(bGroundColor){} 

    //! The destructor
    ~BinaryContourFilter(){}

    //! The filter itself
    void process(Image<T> &srcDest)
    {
	Image<T> tmp(srcDest);
	process (tmp, srcDest);
	return;
    }
	
    void process(const Image<T> &src, Image<T> &dest);

private:
    T fGroundColor;
    T bGroundColor;
}; // BinaryContourFilter

#include <BinaryContourFilter.cpt>

#endif
