/*
   This file is part of the MoCA video library.
   Name: ClipFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   FlipFilter.h  -- implementation of the flip filter

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

*/



#ifndef CLIP_FILTER_H
#define CLIP_FILTER_H

#include <FilterBase.h>
#include <SampleBlock.h>

/*
  Clip the range of the image values.
*/

template<class T>
class ClipFilter: public FilterBase<T>
{
public:
    ClipFilter (T lowClip, T highClip)
    {
		low =lowClip;
		high=highClip;
		if (low>high) {
			cerr << "ClipFilter:: invalid clip values!" << endl;
			exit(1);
		} // it
    } //  constructor

    //! Apply clip filter to src, write out into dest
    void process (const SampleBlock<T>& src, SampleBlock<T>& dest);

    //! Apply clip filter to src, write into same image
    void process (SampleBlock<T>& srcDest);

private:
    T low;
	T high;
}; // ClipFilter

#include <ClipFilter.cpt>

#endif

