/*
    ContrastFilter.cpp     enhance contrast in images
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#include<ContrastFilter.h>

/*!
  \param src source image
  \param dest destination image with increased or reduced contrast
 */
void
ContrastFilter::
process(const Image<uint8> &src, Image<uint8> &dest) 
{
	ImageSize size = src.getSize();
	dest.resize(size);
	for (int32 b=0; b<size[DIM_BAND]; b++) {
		for (int32 y=0; y<size[DIM_Y]; y++) {
			for (int32 x=0; x<size[DIM_X]; x++) {
				ieee64 v = src(x,y,b);
				
				if (v>128.0) {
					v = v + (1.0 - ABS((v-192.0)/64.0))*contrastValue;
				} else {
					v = v - (1.0 - ABS((v-64.0)/64.0))*contrastValue;
				} // else


				if (v>255) v=255;
				if (v<0)   v=0;
				dest(x,y,b) = (uint8)v;
			} // for
		} // for
	} // for
	return;
} // process

