/*
   This file is part of the MoCA video library.
   Name: DerivationFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


/* -*- C++ -*-

   DerivationFilter.h  -- implementation of a lien routine

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   $Id: 
*/



#ifndef DERIVATION_FILTER_H
#define DERIVATION_FILTER_H

#include <MocaSignal.h>
#include <Image.h>
#include <MocaTypes.h>
#include <ConvolutionFilter.h>

/*
  int radius           : radius of the derivation filter
  Directions direction : direction of the filter, e.g. DIR_X, DIR_Y
                         TODO: does DIR_XY make sense?
  Image<T> src         : source image
  Image<T> dest        : destination image
                        
*/

template<class T>
class DerivationFilter : public FilterBase<T>
{
public:
    DerivationFilter(int32 maskSize, Directions direction)
    {
		checkValue (maskSize % 2, "DerivationFilter: Can only process odd mask sizes.");
		radius = maskSize/2;

        this->direction = direction;
    };
    ~DerivationFilter () {};

    //! draw line
    void process(SampleBlock<T>& srcDest);
    void process(const SampleBlock<T> &src, SampleBlock<T>&dest);

private:
    int radius;
    Directions direction;
    
}; // DerivationFilter

#include <DerivationFilter.cpt>

#endif

