/*
   This file is part of the MoCA video library.
   Name: DifferenceFilter.h
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: DifferenceFilter.h,v 1.1 2004/02/17 11:53:52 kopf Exp $

*/

#ifndef DIFFERENCE_FILTER_H
#define DIFFERENCE_FILTER_H

#include <math.h>

#include <MocaExceptions.h>
#include <GetType.h>
#include <SampleBlock.h>

/*!

  \brief calculate the difference of two images 

  This filter takes two images and calculates the difference for each
  pixel, writing it into the third image.
  
 */
class DifferenceFilter
{
public:
    //! the default constructor
    DifferenceFilter() {};

    //! destructor
    ~DifferenceFilter() {};

    //! the filter
    void process(const SampleBlock<uint8> &src1, const SampleBlock<uint8> &src2, SampleBlock<uint8> &diff);

    //! the filter
    void process(const SampleBlock<uint8> &src1, SampleBlock<uint8> &srcDiff);

private:

    //! size of mask for tolerant diff
//    int masksize;
}; // DifferenceFilter

#endif
