/*
   This file is part of the MoCA video library.
   Name: FaceMaskFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   FaceMaskFilter.h  -- creates a mask with possible face positions
   
   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de
*/



#ifndef FACE_MASK_FILTER_H
#define FACE_MASK_FILTER_H

// C++ includes
#include <iostream>
using namespace std;

// MoCA includes
#include <Image.h>
#include <AndFilter.h>
#include <BinarizeFilter.h>
#include <CannyFilter.h>
#include <DilationFilter.h>
#include <EdgeDetectionFilter.h>
#include <GrayscaleFilter.h>
#include <OpeningFilter.h>


/*
  \warning src reference to input image, must be RGB with uint8, the destination image is a grayscale image
 */


class FaceMaskFilter
{
public:
	static const int FACE_MASK_COLOR;
	static const int FACE_MASK_EDGES;
	static const int FACE_MASK_COLOREYES;

    FaceMaskFilter  (int mode = (FACE_MASK_COLOR | FACE_MASK_EDGES)) 
	{
		this->mode = mode;
	};
    ~FaceMaskFilter () {};
    
    //! Apply the face mask filter to src and write to dest
    //! @param src reference to input image, must be RGB with uint8
    void process (Image<uint8>& src, Image<uint8>& dest);
    
    //! Apply the face mask filter to src, write into same image
    //! @param srcDest reference to input image, must be RGB with uint8
    void process (Image<uint8>& srcDest);

private:
    void analyseColor (Image<uint8>& src, Image<uint8>& dest);
    void analyseEdges (Image<uint8>& src, Image<uint8>& dest);
    void analyseColorEyes(Image<uint8>& src, Image<uint8>& dest);

    int mode;
    
}; // FaceMaskFilter

#endif





