/*
   This file is part of the MoCA video library.
   Name: FillFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#ifndef FILL_FILTER_H
#define FILL_FILTER_H

// C++ includes
#include <stack>

// MoCA includes
#include <MocaEnum.h>
#include <Image.h>


class FillFilter
{
public:
    FillFilter () {
		init (N4, 0, 0, false, 0);
	};
    FillFilter (Neighbours neighbour) {
		init (neighbour, 0, 0, false, 0);
	}
    FillFilter (Neighbours neighbour, int enlargeHorizontal, int enlargeVertical) 
	{
		init(neighbour, enlargeHorizontal, enlargeVertical, false, 0);
	}

    FillFilter (Neighbours neighbour, int enlargeHorizontal, int enlargeVertical, uint8 imageColor2) 
	{
		init(neighbour, enlargeHorizontal, enlargeVertical, true, imageColor2);
	}
    ~FillFilter (){};

	void process (Image<uint8> &img, ImagePosition firstIp, uint8 color);


private:
	// functions
	void init (Neighbours neighbour, int enlargeHorizontal, int enlargeVertical, bool secondColor, uint8 imageColor2) {
		this->neighbour         = neighbour;
		this->enlargeHorizontal = enlargeHorizontal;
		this->enlargeVertical   = enlargeVertical;		
		this->secondColor       = secondColor;		
		this->imageColor2       = imageColor2;		
	} // init

	void fill8 (Image<uint8> &img, ImagePosition firstIp, uint8 imageColor1, uint8 imageColor2, uint8 newColor);
	void fill4 (Image<uint8> &img, ImagePosition firstIp, uint8 imageColor1, uint8 imageColor2, uint8 newColor);
	void fillD (Image<uint8> &img, ImagePosition firstIp, uint8 imageColor1, uint8 imageColor2, uint8 newColor);

	// variables
	Neighbours neighbour;
	int enlargeHorizontal;
	int enlargeVertical;
	bool secondColor;
	uint8 imageColor2;
};

#endif
