/*
   This file is part of the MoCA video library.
   Name: GaussDerivationFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


/* -*- C++ -*-

   GaussDerivationFilter.h  -- implementation of a lien routine

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   $Id: 
*/



#ifndef GAUSS_DERIVATION_FILTER_H
#define GAUSS_DERIVATION_FILTER_H

// C includes
#include <math.h>


// MoCA includes
#include <MocaSignal.h>
#include <Image.h>
#include <MocaTypes.h>
#include <ConvolutionFilter.h>

/*
  ieee64 sigma         : sigma of the gauss derivation filter
  Directions direction : direction of the filter, e.g. DIR_X, DIR_Y
                         TODO: does DIR_XY make sense?
  Image<T> src         : source image
  Image<T> dest        : destination image
                        
*/

template<class T>
class GaussDerivationFilter : public FilterBase<T>
{
public:
    static const ieee64 PI;
    
    GaussDerivationFilter(ieee64 sigma, Directions direction)
    {
        this->sigma = sigma;
        this->direction = direction;
    };
    ~GaussDerivationFilter () {};

    //! draw line
    void process (SampleBlock<T>& srcDest);
    void process (const SampleBlock<T>& src, SampleBlock<T>& dest);

private:
    ieee64 sigma;
    Directions direction;
    
}; // GaussDerivationFilter

#include <GaussDerivationFilter.cpt>

#endif

