/*
   This file is part of the MoCA video library.
   Name: GaussFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   GaussFilter.h  -- implementation of the gauss filter

   Author:    Gerald Kuehne
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   $Id: GaussFilter.h,v 1.1 2004/02/17 11:53:52 kopf Exp $
*/



#ifndef GAUSSFILTER_H
#define GAUSSFILTER_H

#include <SampleBlock.h>
#include <FilterBase.h>

/*
  \warning only *ODD* mask sizes work correct
 */
template<class T>
class GaussFilter : public FilterBase<T>
{
public:
  //    GaussFilter (sizeType maskSize);
    GaussFilter (int maskSize);
    GaussFilter (ieee64 sigma);
    ~GaussFilter ();

    //! Apply gaussian filter to src, write out into dest
    void process (const SampleBlock<T>& src, SampleBlock<T>& dest);

    //! Apply gaussian filter to src, write into same image
    void process (SampleBlock<T>& srcDest);

    //! Returns the filter mask size
    sizeType getMaskSize() const { return 2 * maskRadius + 1; }

    //! Gives access to the mask entries
    const ieee64* getMaskPtr() { return mask - maskRadius; }

protected:
    //! Filter from src (type T) to dest (type T)
    void filterLineSD (const T* srcP,  const offsetType* srcOffsetP,
                     T* destP, const offsetType* destOffsetP,
                     sizeType size);

    //! Filter from src (type T) to dest (type T)
    void filterLineD (T* srcP,  const offsetType* srcOffsetP,
                     T* destP, const offsetType* destOffsetP,
                     sizeType size);

    //! the gaussian mask (e.g. 1 2 1)
    ieee64* mask;

    ieee64 sigma;
    
    //! half of maskSize
    sizeType maskRadius;

    //! temporary storage (needed for inplace filtering)
    T* srcLineS;
    T* srcLineD;
}; // GaussFilter

#include <GaussFilter.cpt>

#endif

