/*
   This file is part of the MoCA video library.
   Name: GradientFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   GradientFilter.h  -- samples a SampleBlock down

   Author:    Gerald Kuehne
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   $Id: GradientFilter.h,v 1.1 2004/02/17 11:53:52 kopf Exp $
*/



#ifndef GRADIENTFILTER_H
#define GRADIENTFILTER_H

// MoCA includes
#include <MocaExceptions.h>
#include <GaussFilter.h>
#include <Video.h>


/*!
  \brief Provides a gradient filter for an image or a video

  The class provides a function for calculating the spatial gradient of an
  image or a video. The calculation is performed after the following formula

  \f[
  G = \frac{1}{1+|\nabla I_\sigma|^2/\lambda^2}
  \f]

  where \f$I\f$ denotes the source sample block, \f$\sigma\f$ means
  that the source image is smoothed by a Gauss filter of standard deviation
  \f$\sigma\f$.

*/

template<class T>
class GradientFilter 
{
public:
    //! The constructor
    GradientFilter(sizeType, ieee64);
    //! The destructor
    ~GradientFilter() {}

    //! calculate the gradient filter
    void process(const Image<T>& src, Image<ieee64>& dest);

    //! calculate the gradient filter based on a video
    void process(const Video<T>& src, Video<ieee64>& dest);


protected:
    //! The size of the Gaussian mask
    sizeType maskSize;

    //! Contrast parameter lambda
    ieee64 lambda;
}; // GradientFilter

#include <GradientFilter.cpt>

#endif

