/*
   This file is part of the MoCA video library.
   Name: MeanFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   MeanFilter.h  -- implementation of the mean filter

  
*/

#ifndef MEANFILTER_H
#define MEANFILTER_H

//Moca includes
#include <Image.h>
#include <GetType.h>
#include <MocaError.h>

template<class T>
class MeanFilter
{
public:
  MeanFilter (int maskSize);
  ~MeanFilter ();
  
  enum direction { HORIZONTAL, VERTICAL, BOTH };
  //! Apply mean filter to src, write out into dest
  void process (const Image<T>& src, Image<T>& dest, direction d);
  
  //! Apply mean filter to src, write into same image
  void process (Image<T>& srcDest, direction d);
  
  //! Returns the filter mask size
  int getMaskSize() const { return maskSize; }

protected:
  //! apply mean filter in horizontal direction
  void horizontal (const Image<T> &src, Image<T> &dest, const ImageSize &size);

  //! apply mean filter in vertical direction
  void vertical(const Image<T> &src, Image<T> &dest, const ImageSize &size);

  //! apply mean filter in both directions (corresponds to quadratic mask)
  void both(const Image<T> &src, Image<T> &dest, const ImageSize &size);

  //! filter mask size, must be odd
  int maskSize;
 
}; // MeanFilter

#include <MeanFilter.cpt>

#endif
