/*
   This file is part of the MoCA video library.
   Name: MedianFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


/* -*- C++ -*-

   MedianFilter.h  -- filters the median.

  This filter is a median filter. It takes one sampleblock as argument and
  results in a second sanmpleblock. 
       
   
   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de
*/



#ifndef MEDIAN_FILTER_H
#define MEDIAN_FILTER_H

// C++ includes
//#include<list>
//#include<string>
//#include<iterator>
//#include<iostream>
//using namespace std;

// MoCA includes
#include <SampleBlock.h>
#include <GetType.h>


template<class T>
class MedianFilter
{
public:
    //! the default constructor
    MedianFilter(int radius) {
		this->radius = radius;
    }

    //! the filter
    void process(SampleBlock<T> &srcDest);
    void process(const SampleBlock<T> &src, SampleBlock<T> &dest);

    //! process a signal and get median value
    void process(const Signal<T> &src, T &val);


    //! the destructor
    ~MedianFilter() {};

    int radius;
}; // MedianFilter


#include <MedianFilter.cpt>

#endif
