/*
   This file is part of the MoCA video library.
   Name: OpeningFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#ifndef OPENING_FILTER_H
#define OPENING_FILTER_H

// MoCA includes
#include <MocaExceptions.h>
#include <ImageFilterBase.h>
#include <ErosionFilter.h>
#include <DilationFilter.h>


/*!

  \brief An opening Filter

  This filter smoothes the contour of an image (erosion, dilation). 

 */
template<class T>
class OpeningFilter : public ImageFilterBase<T>
{
public:
    //! the default constructor
    /*!
      \param width the width of the mask, must be odd
     */
    OpeningFilter(int width = 5);

    //! the filter
    void process(Image<T> &srcDest);

    //! the filter
    void process(const Image<T> &src, Image<T> &dest);

    //! the destructor
    ~OpeningFilter();

protected:

private:
    //! mask width
    int width;
}; // OpeningFilter


#include <OpeningFilter.cpt>

#endif
