/*
   This file is part of the MoCA video library.
   Name: StructureTensorFilter.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

  Implementation of the structure tensor

   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: StructureTensorFilter.h,v 1.1 2004/02/17 11:53:52 kopf Exp $

*/

#ifndef STRUCTURETENSORFILTER_H
#define STRUCTURETENSORFILTER_H

// C includes
#include <assert.h>
#include <assert.h>

//#include <FilterBase.h>
#include <GaussFilter.h>

/*!
  Structure tensor
*/
template<class T>
class StructureTensorFilter 
{
public:
    //! the constructor
    StructureTensorFilter(sizeType gaussFilterSize, float noiseLevel)
            : gaussFilterSize(gaussFilterSize),
              noiseLevel(noiseLevel) {}

    //! the destructor
    ~StructureTensorFilter() {}

    //! Apply structure tensor to src, write out into dest
    void process (const Video<T>& src, Video<float>& dest);
    //! Apply stucture tensor with displacement field
    void process (const Video<T>& src, const Video<float>& displacement, Video<float>& dest);

private:
    sizeType gaussFilterSize;
    float  noiseLevel;

    //! stuff for eigenvalue calculation
    enum { N = 3 }; // size of the tensor (3x3)
    enum { MAXITERATIONS = 1000 };

    T interpolate(const Video<T>&, ieee64, ieee64, indexType);
    void sortEigenvalues (float [3], int&, int&, int&);
    void calculateEigenvalues(float [N][N], float [N], float [N][N]);
    bool checkEigenvalues (float [N][N], float [N], float [N][N]);
}; // StructureTensor


#include <StructureTensorFilter.cpt>

#endif

