/*
   This file is part of the MoCA video library.
   Name: Color.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   Color.h  -- defines a struct Color and several default colors

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   $Id: 
*/



#ifndef COLOR_H
#define COLOR_H

#include <MocaTypes.h>

struct Color
{
	Color ()
	{
		init (0,0,0);
	} // Color
	Color (uint8 c0, uint8 c1, uint8 c2)
	{
		init (c0, c1, c2);
	} // Color
	Color (uint8 c0)
	{
		init (c0, 0, 0);
	} // Color
	/*
	Color (Color &other) 
	{
		init (other.c[0], other.c[1], other.c[2]);
	} // Color
	*/
	Color (const Color &other) 
	{
		init (other.c[0], other.c[1], other.c[2]);
	} // Color

	
    Color& operator=(const Color &other) {
		if (this != &other) {
			c[0] = other.c[0];
			c[1] = other.c[1];
			c[2] = other.c[2];
		}
        return *this;
    } // operator=

    static Color toYUV(const Color &rgb) {
		Color c;
		c.c[0] = (( 65*rgb[0] +  129*rgb[1] +  24*rgb[2])>>8)+16;
		c.c[1] = ((-37*rgb[0] +  -74*rgb[1] + 112*rgb[2])>>8)+128;
		c.c[2] = ((112*rgb[0] +  -93*rgb[1] + -18*rgb[2])>>8)+128;
        return c;
    } // operator=
	

	uint8 operator[](sizeType i) {
		return c[i];
	} // operator[]
	const uint8 operator[](sizeType i) const
	{	
		return c[i];
	} // operator[]

	void init (uint8 c0, uint8 c1, uint8 c2) 
	{
		c[0] = c0;
		c[1] = c1;
		c[2] = c2;
	}
	uint8 c[3];

	// static variables
	const static Color BLACK;
	const static Color GRAY;
	const static Color LIGHT_GRAY;
	const static Color WHITE;

	const static Color GREEN;
	const static Color BLUE;
	const static Color RED;
	const static Color ORANGE;
	const static Color YELLOW;

};


#endif

