/*
    DrawCircle.cpp     draw a circle
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#include<DrawCircle.h>


void
DrawCircle::process (const ImagePosition &p, const int32 radius, const Color &c, const Image<uint8>& src, Image<uint8>& dest)
{
	dest=src;
	process(p, radius, c, dest);
    return;
} // process


void
DrawCircle::process (const ImagePosition &p, const int32 radius, const Color &c, Image<uint8>& srcDest)
{
	// draw a circle to srcDest
	ImageSize size = srcDest.getSize();
	int32 sx = size[DIM_X];
	int32 sy = size[DIM_Y];
	int32 nSteps = (int32)(ceil(sqrt((ieee64)radius*radius/2))+1);
	
	int32* step = new int32[nSteps];
	for (int32 i=0; i<nSteps; i++) {
		step[i] = (int32)(radius-(sqrt((ieee64)radius*radius-i*i)) + 0.5);
	} // for
	
	for (int32 band=0; band<size[DIM_BAND]; band++) {
		for (int32 i=0; i<nSteps; i++) {
			int32 px = p[DIM_X]+i;
			int32 py = p[DIM_Y]-radius+step[i];
			if ((px>=0) && (px>=0) && (px<sx) && (py<sy)) srcDest(px,py,band) = c[band];
			
			px = p[DIM_X]-i;
			if ((px>=0) && (px>=0) && (px<sx) && (py<sy)) srcDest(px,py,band) = c[band];
			
			py = p[DIM_Y]+radius-step[i];
			if ((px>=0) && (px>=0) && (px<sx) && (py<sy)) srcDest(px,py,band) = c[band];
			
			px = p[DIM_X]+i;
			if ((px>=0) && (px>=0) && (px<sx) && (py<sy)) srcDest(px,py,band) = c[band];
			
			
			px = p[DIM_X]+radius-step[i];
			py = p[DIM_Y]-i;
			if ((px>=0) && (px>=0) && (px<sx) && (py<sy)) srcDest(px,py,band) = c[band];
			
			py = p[DIM_Y]+i;
			if ((px>=0) && (px>=0) && (px<sx) && (py<sy)) srcDest(px,py,band) = c[band];
			
			px = p[DIM_X]-radius+step[i];
			if ((px>=0) && (px>=0) && (px<sx) && (py<sy)) srcDest(px,py,band) = c[band];
			
			py = p[DIM_Y]-i;
			if ((px>=0) && (px>=0) && (px<sx) && (py<sy)) srcDest(px,py,band) = c[band];
		
		} // for
	} // for
	delete[] step;


/*	DrawLine line(c);
	line.process (ImagePosition(p[DIM_X]-5,p[DIM_Y]-5), ImagePosition(p[DIM_X]+5,p[DIM_Y]+5), srcDest);
	line.process (ImagePosition(p[DIM_X]+5,p[DIM_Y]-5), ImagePosition(p[DIM_X]-5,p[DIM_Y]+5), srcDest);
*/
	return;
} // process

