/*
   This file is part of the MoCA video library.
   Name: DrawDot.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   DrawDot.h  -- implementation of a dot routine

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   $Id: 
*/



#ifndef DRAW_DOT_H
#define DRAW_DOT_H

#include <ImagePosition.h>
#include <Image.h>
#include <Color.h>


class DrawDot 
{
public:
    DrawDot(const Color &c = Color::WHITE) {
		this->c=c;
		dotSize = 2;
	};
    DrawDot(int dotSize) {
		this->dotSize = dotSize;
	};

    //! draw line
    void process (ImagePosition p, const Color &c, Image<uint8>& srcDest);
    void process (ImagePosition p, const Color &c, const Image<uint8>& src, Image<uint8>& dest);

    //! draw line
    void process (ImagePosition p, Image<uint8>& srcDest) 
	{
		process (p, c, srcDest);
	}
    void process (ImagePosition p, const Image<uint8>& src, Image<uint8>& dest) 
	{
		process (p, c, src, dest);
	}


    //! draw line
    void process (int32 x, int32 y, const Color &c, Image<uint8>& srcDest)
	{
		process (ImagePosition(x,y), c, srcDest);
	}
    void process (int32 x, int32 y, const Color &c, const Image<uint8>& src, Image<uint8>& dest)
	{
		process (ImagePosition(x,y), c, src, dest);
	}

    //! draw line
    void process (int32 x, int32 y, Image<uint8>& srcDest) 
	{
		process (ImagePosition(x,y), c, srcDest);
	}
    void process (int32 x, int32 y, const Image<uint8>& src, Image<uint8>& dest) 
	{
		process (ImagePosition(x,y), c, src, dest);
	}


private:
	Color c;
	int dotSize;
}; // DrawDot

#endif

