/*
    DrawLine.cpp     draw a line 
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include<DrawLine.h>


void
DrawLine::process (const ImagePosition &p1, const ImagePosition &p2, const Color &c, const Image<uint8>& src, Image<uint8>& dest)
{
	dest=src;
	process(p1, p2, c, dest);
    return;
} // process


void
DrawLine::process (const ImagePosition &p1, const ImagePosition &p2, const Color &c, Image<uint8>& srcDest)
{
	// draw standard line
	draw (p1, p2, c, srcDest);

	// draw thick line
	if (width > 1) {
		int32 left  = (width-1) / 2;
		int32 right = (width-1) - left;

		int32 dx = abs(p1[DIM_X]-p2[DIM_X]);
		int32 dy = abs(p1[DIM_Y]-p2[DIM_Y]);
		if (dx<dy) {
			for (int32 i=1; i<=left; i++) {
				ImagePosition p3 = p1;
				ImagePosition p4 = p2;
				p3[DIM_X]-=i;
				p4[DIM_X]-=i;
				draw (p3, p4, c, srcDest);
			}
			for (int32 i=1; i<=right; i++) {
				ImagePosition p3 = p1;
				ImagePosition p4 = p2;
				p3[DIM_X]+=i;
				p4[DIM_X]+=i;
				draw (p3, p4, c, srcDest);
			}
		} else {
			for (int32 i=1; i<=left; i++) {
				ImagePosition p3 = p1;
				ImagePosition p4 = p2;
				p3[DIM_Y]-=i;
				p4[DIM_Y]-=i;
				draw (p3, p4, c, srcDest);
			}
			for (int32 i=1; i<=right; i++) {
				ImagePosition p3 = p1;
				ImagePosition p4 = p2;
				p3[DIM_Y]+=i;
				p4[DIM_Y]+=i;
				draw (p3, p4, c, srcDest);
			}
		} // else

	} // if
	return;
} // process



void 
DrawLine::draw (const ImagePosition &p1, const ImagePosition &p2, const Color &c, Image<uint8>& srcDest)
{
	int32 w = srcDest.getSize()[DIM_X];
	int32 h = srcDest.getSize()[DIM_Y];

	// draw line 
    int32 i, b;
	int x1 = p1[DIM_X];
	int x2 = p2[DIM_X];
	int y1 = p1[DIM_Y];
	int y2 = p2[DIM_Y];
	int nb = (srcDest.getSize())[DIM_BAND];
	
    // case 1: x1 == x2
    // ----------------
    if (x1 == x2) {
        if (y2 > y1) {
            for (i=y1; i<=y2; i++) {
	            for (b=0; b<nb; b++) {
					if ((x1>=0) && (x1<w) && (i>=0) && (i<h))
						srcDest (x1, i, b) = c[b];
				} // for
            } // for
        } // if
        else {
            for (i=y2; i<=y1; i++) {
	            for (b=0; b<nb; b++) {
					if ((x1>=0) && (x1<w) && (i>=0) && (i<h))
		                srcDest (x1, i, b) = c[b];
				} // for
            } // for
        } // else
    } // if
    else {
		
        // case 2: y1 == y2
        // ----------------
        if (y1 == y2) {
            if (x2 > x1) {
                for (i=x1; i<=x2; i++) {
					for (b=0; b<nb; b++) {
						if ((i>=0) && (i<w) && (y1>=0) && (y1<h))
							srcDest (i, y1, b) = c[b];
					} // for
                } // for
            } // if
            else {
                for (i=x2; i<=x1; i++) {
		            for (b=0; b<nb; b++) {
						if ((i>=0) && (i<w) && (y1>=0) && (y1<h))
			                srcDest (i, y1, b) = c[b];
					} // for
                } // for
            } // else
        } // if
        else {
			
            // case 3: (x1 != x2) && (y1 != y2)
            // -------------------------------
            double j, delta;
            if (abs(x2-x1) > abs(y2-y1)) {
				
                // Fall 3a: (x2-x1) > (y2-y1)
                if (x2 > x1) {

					// case: x2 > x1
                    j = y1 + 0.5;
                    delta = (double) (y2-y1) / (x2-x1);
                    for (i=x1; i<=x2; i++) {
			            for (b=0; b<nb; b++) {
							if ((i>=0) && (i<w) && (((int32) j)>=0) && (((int32) j)<h))
								srcDest (i,(int32) j, b) = c[b];
						} // for
                        j += delta;
                    } // for
                } else { 

					// case: x2 < x1
                    j = y2 + 0.5;
					delta = (double) (y2-y1) / (x2-x1);
					for (i=x2; i<=x1; i++) {
						for (b=0; b<nb; b++) {
							if ((i>=0) && (i<w) && (((int32) j)>=0) && (((int32) j)<h))
								srcDest (i,(int32) j, b) = c[b];
						} // for
						j += delta;
					} // for
                } // else
            }  // else: case 3a
            else {
				
                // case 3b: (x2-x1) < (y2-y1)
                if (y2 > y1) {

					// case: y2 > y1
                    j = x1 + 0.5;
                    delta = (double) (x2-x1) / (y2-y1);
                    for (i=y1; i<=y2; i++) {
			            for (b=0; b<nb; b++) {
							if ((((int32) j)>=0) && (((int32) j)<w) && (i>=0) && (i<h))
								srcDest ((int32) j, i, b) = c[b];
						} // for
                        j += delta;
                    } // for
                } else { 
					
					// case: y2 < y1
                    j = x2 + 0.5;
					delta = (double) (x2-x1) / (y2-y1);
					for (i=y2; i<=y1; i++) {
						for (b=0; b<nb; b++) {
							if ((((int32) j)>=0) && (((int32) j)<w) && (i>=0) && (i<h))
								srcDest ((int32) j, i, b) = c[b];
						} // for
						j += delta;
					} // for
                } // else
            }  // else: case 3b
        } // else: case 3
    } // else: case 2

	return;
}

