/*
   This file is part of the MoCA video library.
   Name: CreateMocaPath.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#ifndef CREATE_MOCA_PATH_H
#define CREATE_MOCA_PATH_H

// C++ includes
#include <string>
#include <iostream>
using namespace std;

// MoCA includes
#include <FileHandling.h>

// needed for stat
//#include <sys/types.h> 
//#include <sys/stat.h>  

/*
	CreateMocaPath (filename)

	(1) Constructor analyses filename and defines the variables:
			longFilename   : just the parameter of the constructor
			filename	   : the filename without teh path
			path		   : the path of the filename
			destinationPath: the path, where the MoCA resulst are stored
		Example:
			filename = "/opt/test/test.mpg"
			==>	longFilename   : "/opt/test/test.mpg"
				filename	   : "test.mpg"
				path		   : "/opt/test/"
				destinationPath: "/opt/test/test_index/"

    (2) Constructor creates the directory destinationPath.
		Unlike "mkdir" if any of the parent directories of the specified 
		directory do not exist, it creates them as well.

*/


class CreateMocaPath;
std::ostream& operator<< (ostream&, const CreateMocaPath&);


class CreateMocaPath
{
	static const string MOCA_PATH;
	
public:

	// constructor
	CreateMocaPath (const string &pathFile, bool createDirectory=true) {
		this->pathFile        = pathFile;
		this->createDirectory = createDirectory;
		file = "";
		path = "";		
		destinationPath = "";

		setVariables ();
		if (createDirectory) {
			FileHandling::createDirectory (destinationPath, false);
			//createPath();
		} // if

	} // CreatePath


	// variables
	bool createDirectory;
	string file;
	string pathFile;
	string path;		
	string destinationPath;		

public:
	// if a path contains spaces, e. g. name='C:\test videos\mpg\',
	// change name to name='"C:\test videos\mpg\"'
	void validatePath (string &name);

private:
	void setVariables ();
	//void createPath ();

}; // CreateMocaPath


#endif
