/*
   This file is part of the MoCA video library.
   Name: FileHandling.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   file.h  -- short description

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

*/

#ifndef FILE_HANDLING
#define FILE_HANDLING


// MoCA includes

#include <MocaTypes.h>

#include <MocaExceptions.h>




// C includes for directory commands
#include <sys/types.h>  // required for stat
#include <sys/stat.h>   // required for stat

#ifdef COMPILER_MSVC
#include <direct.h>		
#include <io.h>		
#include <sys/types.h>
#include <sys/stat.h>

#else
#include <dirent.h>	
#include <unistd.h>	
#endif



// C++ includes
#include <iomanip>
#include <sstream>
#include <iostream>
#include <string>
#include <fstream>
using namespace std;




/*
	Class handles basic file io operations, e. g. deletes files, creates directories

*/


class FileHandling
{
public:
    static void deleteFile (const string &filename, bool displayMessage=true);
	static void renameFile (const string &filenameSrc, const string &filenameDest, bool displayMessage=true);
	static void copyFile (const string &filenameSrc, const string &filenameDest, bool displayMessage=true);
	static void createDirectory (const string &directoryName, bool displayMessage=true);
	static void deleteDirectory (const string &pathName, bool displayMessage);

	static void convertToValidUnixFilename (const string &srcFilename, string &destFilename);
	static void convertToValidUnixFilename (string &filename)
	{
	    string tmp = filename;
	    convertToValidUnixFilename(tmp, filename);
	    return;
	}

	static void convertToValidFilename (string &filename);
	static void convertToUnixFilename (string &filename);

	static void getCurrentDirectory (string &currentDirectory);
	static void setCurrentDirectory (const string &currentDirectory);

	static bool fileExists (const string &pathFilename);
	static void getFileSize (string &pathFilename, int32 &sizeInBytes);


	// get expension of filename, e. g. "test.WAV" --> extension="wav", beforeExtension="test"
	static void getFileExtension (string &pathFilename, string &extension);
	static void getFileExtension (string &pathFilename, string &extension, string &beforeExtension);

	// convert string to upper case
	static void upper (string &s)
	{
		for (size_t i=0; i< s.length(); i++) {
			s[i] = toupper(s[i]);  
		} // for
	} // upper

	// convert string to lower case
	static void lower (string &s)
	{
		for (size_t i=0; i< s.length(); i++) {
			s[i] = tolower(s[i]);  
		} // for
	} // lower

	static string getNumberedFilename (const string &prefix, int32 frame, const string &suffix)
	{
		stringstream frameNumberStream;
		frameNumberStream << setw(6) << setfill('0') << frame;
		string frameNumberString = frameNumberStream.str();


		string s = prefix + frameNumberString + suffix;
		return s;
	} // getNumberedFilename


private:

};


#endif

