/*
    FileList.cpp     get a list of files in directory
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <FileList.h>

// static variable
const int FileList::RECURSE_DEPTH = 7;


void 
FileList::show(vector<FileInfo>& fileInfo)
{
    vector<FileInfo>::iterator it;
    for (it = fileInfo.begin(); it != fileInfo.end(); it++) {
        cout << (*it).name << endl;
    } // for    
	return;
} // show


void 
FileList::sort (vector<FileInfo>& fileInfo)
{
	// convert to list
	list<FileInfo> tmp;
	for (int32 i=0; i<fileInfo.size(); i++) {
		tmp.push_back(fileInfo[i]);
	} // for

	// sort
	tmp.sort();

	// store in vector
    list<FileInfo>::iterator it;
	int32 i=0;
    for (it = tmp.begin(); it != tmp.end(); it++) {
        fileInfo[i]=(*it);
		i++;
    } // for    
	return;
} // show


// TODO  throw exception
void
FileList::process(const string &fileId, vector<FileInfo>& fileList)
{
    this->fileId = fileId;
    fileList.clear();


    // define path
#ifdef _WIN32
    if ((path == "./") || (path == "./")) {
        // get current working directory
        char buffer[4096];
        if( _getcwd( buffer, 4096) == NULL ) {
			cerr << "FileList::process()  Unable to get current directory "<< endl;
			exit(1);
		} else {
			path = buffer;
		} // else
    } // if
#endif

    long p;
    for (p=0; p<(long)path.size(); p++) {
        if (path[p] == '\\') {
            path[p] = '/';
        } // if
    } // for
    if (path[path.size()-1] != '/') {
        path += "/";
    } // if

    
    // get prefix and suffix
	p = fileId.find_last_of('*');
	if (p != (long)string::npos) {
        prefixId = fileId.substr(0, p);
		suffixId = fileId.substr(p+1);
    } // if
	
    
	// change directory
#ifdef _WIN32
	if( _chdir(path.c_str())) {					
		cerr << "FileList::process()  Unable to locate the directory: "<< path << endl;
		exit(1);
	} // if
#else
	DIR* d = opendir(path.c_str());
	if (d == NULL) { 
		cerr << "FileList::process()  Unable to locate the directory: "<< path << endl;
		exit(1);
	} // if
#endif

	// read files
	recurseDirectory (path, fileList, 1);

#ifndef _WIN32
    // close directory (linux)
    closedir(d);
#endif
    

	FileHandling::setCurrentDirectory (currentDirectory);
	return;
} // init



void
FileList::unselect(const string &fileId, vector<FileInfo>& fileList)
{

	// define prefix and suffix
    string tmpPrefix = "";
    string tmpSuffix = "";
	uint32 p=0;
	p = fileId.find_last_of('*');
	if (p != string::npos) {
        tmpPrefix = fileId.substr(0, p);
		tmpSuffix = fileId.substr(p+1);
    } // if

	// to lower
	for (size_t i=0; i< tmpPrefix.length(); i++) {
		tmpPrefix[i] = tolower(tmpPrefix[i]);  
	} // for
	for (size_t i=0; i< tmpSuffix.length(); i++) {
		tmpSuffix[i] = tolower(tmpSuffix[i]);  
	} // for


	// remove all files from vector, which match the fileID
	vector<FileInfo>::iterator it = fileList.begin();
	while (it!=fileList.end()) {
		string name = (*it).shortName;

		// to lower
		for (size_t i=0; i< name.length(); i++) {
			name[i] = tolower(name[i]);  
		} // for

		bool valid = true;
        if ((!tmpPrefix.empty()) && (name.find (tmpPrefix) != 0)) {
			valid=false;
		} // if
        if ((!tmpSuffix.empty()) && (name.rfind(tmpSuffix) != name.size() - tmpSuffix.size())) {
			valid=false;
		} // if

		// remove item
		if (valid) {
			fileList.erase(it);
			it = fileList.begin();
		} else {
			it++;
		} // else

	} // while

	return;
} // init


 
void
FileList::select(const string &fileId, vector<FileInfo>& fileList)
{

	// define prefix and suffix
    string tmpPrefix = "";
    string tmpSuffix = "";
	uint32 p=0;
	p = fileId.find_last_of('*');
	if (p != string::npos) {
        tmpPrefix = fileId.substr(0, p);
		tmpSuffix = fileId.substr(p+1);
    } // if

	// to lower
	for (size_t i=0; i< tmpPrefix.length(); i++) {
		tmpPrefix[i] = tolower(tmpPrefix[i]);  
	} // for
	for (size_t i=0; i< tmpSuffix.length(); i++) {
		tmpSuffix[i] = tolower(tmpSuffix[i]);  
	} // for


	vector<FileInfo> newList;
	// add all files from vector, with matching the fileID
	vector<FileInfo>::iterator it = fileList.begin();
	while (it!=fileList.end()) {
		string name = (*it).shortName;

		// to lower
		for (size_t i=0; i< name.length(); i++) {
			name[i] = tolower(name[i]);  
		} // for

		bool valid = true;
        if ((!tmpPrefix.empty()) && (name.find (tmpPrefix) != 0)) {
			valid=false;
		} // if
        if ((!tmpSuffix.empty()) && (name.rfind(tmpSuffix) != name.size() - tmpSuffix.size())) {
			valid=false;
		} // if

		// remove item
		if (valid) {
		    newList.push_back(*it);
		} // if
		it++;

	} // while

	fileList = newList;
	return;
} // init


void 
FileList::
addFile (string& path, string& fn, vector<FileInfo>& allFiles)
{
    FileInfo temp;	
    temp.name = path + fn;

	// to lower
	string tmpSuffix = suffixId;
	for (size_t i=0; i< tmpSuffix.length(); i++) {
		if (isupper(tmpSuffix[i])) {
			tmpSuffix[i] = tolower(tmpSuffix[i]);  
		} // if
	} // for
	

    // check prefix
    if (prefixId.empty() || (fn.find (prefixId) == 0)) {
        // check prefix

		string tmp = temp.name;
		for (size_t i=0; i< tmp.length(); i++) {
			if (isupper(tmp[i])) {
				tmp[i] = tolower(tmp[i]);  
			} // if
		} // for

        if ((suffixId.empty()) || 
//			(temp.name.rfind(suffixId) == temp.name.size() - suffixId.size())) {
			(tmp.rfind(tmpSuffix) == temp.name.size() - suffixId.size())) {
            temp.path = path;
            temp.shortName = fn;
            allFiles.push_back(temp);
        } // if
    } // if   
    return;
} // addFile


void 
FileList::
recurseDirectory (string& path, vector<FileInfo>& allFiles, int startDepth)
{ 
    // stop recursion
	if (startDepth+1 > depth) {
		return;
	} // if
	
    string fn;
#ifdef _WIN32	
	// check if path exists
	if( !_chdir(path.c_str())) {					
		
		struct _finddata_t data;
		long dStatus;
		string newPath;
		
		if( (dStatus = _findfirst("*.*", &data)) != -1L ) { 
			fn = data.name;
			if (data.attrib & _A_SUBDIR) {
				if ((fn != ".") && (fn != "..")) {
					newPath = path + fn + "/";
					if (recurse) {
						recurseDirectory (newPath, allFiles, startDepth+1);
					} // 
				} // if
			} else {
				addFile (path, fn, allFiles);
			} // else
						
			while( _findnext( dStatus, &data ) == 0 ) {
				fn = data.name;
				if (data.attrib & _A_SUBDIR) {
					if ((fn != ".") && (fn != "..")) {
						newPath = path + fn + "/";
						if (recurse) {
							recurseDirectory (newPath, allFiles, startDepth+1);
						} // if
					} // if
				} else {
                    addFile (path, fn, allFiles);     
				} // else
			} // while
		} // if
		_findclose(dStatus);
	} // if

    
#else // LINUX, SUN
	
    DIR* d = opendir(path.c_str());
    if (d != NULL) { 
		
		string newPath;
		struct dirent* dr;
		while ((dr = readdir(d))) {
			fn = dr->d_name;
			
			newPath = path + fn + "/";
			DIR* isDir = opendir(newPath.c_str());
			if (isDir != NULL) {
				if ((fn != ".") && (fn != "..")) {
					if (recurse) {
						recurseDirectory (newPath, allFiles, startDepth+1);
					} // if
				} // if
			} else {
		addFile (path, fn, allFiles);     
		//string temp = path+fn;
                //addFile (temp, allFiles);     
			} // else
		} // while
	} // if
#endif
	return;
} // recurseDirectory



void 
FileList::
deleteFiles (vector<FileInfo>& fileList)
{
	// delete all files in fileList
	vector<FileInfo>::iterator it;
	for (it=fileList.begin(); it!=fileList.end(); it++) {
		string filename = (*it).name;
		FileHandling::deleteFile (filename);
	} // for

	return;
} // deleteFiles

