/*
   This file is part of the MoCA video library.
   Name: FileReader.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   FileReader.h  -- read a file and store it in memory 

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

*/

#ifndef FILE_READER
#define FILE_READER


// C++ includes
#include <vector>
using namespace std;


// MoCA includes
#include <OpenFile.h>
#include <FileHandling.h>

// C includes
#ifdef COMPILER_MSVC
#include <fcntl.h>
#endif




/*
  Class reads a file and stored it in memory
*/


class FileReader
{

public:
    //! the default constructor
    FileReader(const string &filename) 
    {
		this->filename=filename;
		buf=NULL;
		size = 0;
		init();       
	}
    
	// destructor	
	~FileReader ()
    {
		close();
	}
	void getBufPtr (char *bufPtr) 
	{
		bufPtr = buf;
		return;
	}
	void convertToText   (vector<string> &textLines);
	void convertToNumber (vector<ieee64> &numbersLines)
	{
		numbersLines.clear();
		vector<string> textLines;
		convertToText(textLines);
		int32 n = textLines.size();
		for (int32 i=0; i<n; i++) {
			string s = textLines[i];
			ieee64 v = atof (s.c_str());
			numbersLines.push_back(v);
		}
	}


private:
    // Interface
	void init (); 
	void close ()
	{
		if (buf != NULL) {
			delete[] buf;
		} // if
		return;
	}
    


    // variables
	int32 size;
	string filename;
	char *buf;
};


#endif

