/*
   This file is part of the MoCA video library.
   Name: FormatedText.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@pi4.informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#ifndef FORMATED_TEXT_H
#define FORMATED_TEXT_H


// C++ includes
#include <vector>
#include <string>
#include <fstream>
#include <iomanip>
#include <iostream>
using namespace std;

// MoCA includes
#include <FileHandling.h>
#include <MocaTypes.h>
#include <MocaExceptions.h>

struct FormatedTextItem
{
	FormatedTextItem () {
		tag      = "";
		dataName = "";
		data     = "";
		status   = 0;
	}

    FormatedTextItem& operator=(const FormatedTextItem &other) {
		if (this != &other) {
			tag      = other.tag;
			dataName = other.dataName;
			data     = other.data;
			status   = other.status;
		} // if
        return *this;
    } // operator=

	string tag;
	string dataName;
	string data;
	int8 status;
};

std::ostream& operator<< (ostream &, const FormatedTextItem&);
std::ostream& operator<< (ostream &, const vector<FormatedTextItem>&);


class FormatedText
{
public:

    //! open formated text file
    FormatedText(const string &filename); 

    void process (vector<FormatedTextItem> &data);
    void findData (string &tagName, vector<FormatedTextItem> &formatedTextData, FormatedTextItem &data);
	void removeFirstIdentifier (string &id, vector<FormatedTextItem> &FormatedTextData);
	void findDataList (FormatedTextItem &firstSearchItem, FormatedTextItem &secondSearchItem, vector<FormatedTextItem> &srcList, vector<FormatedTextItem> &destList);

private:
	bool openFile(ifstream &ifs);
	void createList (ifstream &ifs, vector<FormatedTextItem> &formatedTextData);
	void aggregateList (vector<FormatedTextItem> &formatedTextData);


	string filename;
};


#endif




