/*
   This file is part of the MoCA video library.
   Name: IJpegFormatStream.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: IJpegFormatStream.h,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

#ifndef I_JPEG_FORMAT_STREAM_H
#define I_JPEG_FORMAT_STREAM_H

// MoCA includes
#include <MocaTypes.h>

#ifdef HAVE_LIBJPEG

// jpeg includes
extern "C" {
#include <jpeglib.h>
#include <jerror.h>
}

// C includes
#include <limits.h>

// MoCA includes
#include <IImageFormatStream.h>
#include <GetType.h>

/*!
  pgm format specific stream
*/
template<class T>
class IJpegFormatStream : public IImageFormatStream<T>
{
public:
    //! the default constructor
    IJpegFormatStream(istream &is) : IImageFormatStream<T>(is) {};

    //! read an image
    void read(Image<T> &) throw (MocaException);

    //! check if "is" is of given format
    bool checkFormat();

private:


  //! user defined error handler
  struct my_error_mgr {
    struct jpeg_error_mgr pub;	/* "public" fields */

    jmp_buf setjmp_buffer;	/* for return to caller */
  };

  typedef struct my_error_mgr * my_error_ptr;


  //! data source manager stuff for using istream
  typedef struct {
    struct jpeg_source_mgr pub;	/* public fields */

    istream* infile;		/* source stream */
    JOCTET* buffer;		/* start of buffer */
    boolean start_of_file;	/* have we gotten any data yet? */
  } istream_source_mgr;

  typedef istream_source_mgr* istream_src_ptr;

  enum { INPUT_BUF_SIZE = 4096 };

    static void jpeg_istream_src (j_decompress_ptr cinfo, istream* infile);
  static void init_source_istream (j_decompress_ptr cinfo);
  static boolean fill_input_buffer_istream (j_decompress_ptr cinfo);
  static void skip_input_data_istream (j_decompress_ptr cinfo, long num_bytes);
  static void term_source_istream (j_decompress_ptr cinfo);
  // use default resync_to_restart routine
  static void my_error_exit (j_common_ptr);
}; // IJpegFormatStream

#include <IJpegFormatStream.cpt>

#endif

#endif
