/*
   This file is part of the MoCA video library.
   Name: IVideoDirectoryStream.cpt
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: IVideoDirectoryStream.cpt,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

/*!

  call standard constructor of base class to prevent loops

 */

#include <IVideoDirectoryStream.h>



inline
IVideoDirectoryStream::IVideoDirectoryStream(const string &uri)
        : IVideoStream(), nfl(NULL)
{
    initialise(uri);
} // IVideoDirectoryStream::IVideoDirectoryStream


/*!

  does not call base class open, not necessary!

 */
inline
void
IVideoDirectoryStream::open(const string &uri)
{
    initialise(uri);
} // IVideoDirectoryStream::open


/*!

  do the work of constructor and open

 */
inline
void
IVideoDirectoryStream::initialise(const string &uri) throw (MocaException)
{

    if (nfl != NULL) {
        throw MocaException("Please close stream IVideoDirectoryStream before opening it.");
    } // if

    nfl = new NumberedFileList(uri);

    if (nfl == NULL) {
        throw MocaException("Fatal memory trouble, lie down and don't move!");
    } // if

} // IVideoDirectoryStream::initialise


/*!
 */
inline
IVideoDirectoryStream::~IVideoDirectoryStream()
{
    finalise();
} // IVideoDirectoryStream::~IVideoDirectoryStream


/*!
 */
inline
void
IVideoDirectoryStream::close()
{
    finalise();
} // IVideoDirectoryStream::close


/*!

  do the work of destructor and close

 */
void
IVideoDirectoryStream::finalise()
{
    if (nfl != NULL) {
        delete nfl;
    } // if

    nfl = NULL;
} // IVideoDirectoryStream::finalise


/*!

  \param start default 0, starting position in computer science matter (0 ist the first!)

 */
inline
void
IVideoDirectoryStream::read(Video<uint8> &dest, int start, int num) throw (MocaException)
{

    // figure out number of images to be in video
    int z;

    if (num == INT_MAX) {
        z = nfl->size() - start;
    } else {
        z = num;
    } // if

    if (z <= 0) {
        throw MocaException("You do not want to load any images into video...");
    } // if

    // load first frame and get dimensions of image, video
    VideoSize vidSize;
    vidSize[DIM_Z] = z;


    IImageStream<uint8> is((*nfl)[start]);
    Image<uint8> img;
    is.read(img);

    ImageSize imgSize = img.getSize();

    vidSize[DIM_X] = imgSize[DIM_X];
    vidSize[DIM_Y] = imgSize[DIM_Y];
    vidSize[DIM_BAND] = imgSize[DIM_BAND];

    dest.resize(vidSize);

    // put first frame into video
    dest.setImage(0, img);

    is.close();

    // load all following images
    for (int i = 1; i < z; i++) {

        is.open((*nfl)[start + i]);
        is.read(img);

        dest.setImage(i, img);

        is.close();
    } // for

} // IVideoDirectoryStream::read


