/*
   This file is part of the MoCA video library.
   Name: IVideoDirectoryStream.h
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file


   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: IVideoDirectoryStream.h,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

#ifndef I_VIDEO_DIRECTORY_STREAM_H
#define I_VIDEO_DIRECTORY_STREAM_H

// C includes
#include <limits.h>

// C++ includes
#include <string>

// MoCA includes
#include <IVideoStream.h>
#include <MocaExceptions.h>
#include <Video.h>
#include <NumberedFileList.h>
#include <IImageStream.h>

/*!

  Input stream for image files

 */

class IVideoDirectoryStream : public IVideoStream
{
public:
    //! the default constructor
    IVideoDirectoryStream() : nfl(NULL) {};

    //! construct stream based on url provided
    IVideoDirectoryStream(const string &);

    //! open an image stream
    void open(const string &);

    //! the destructor
    ~IVideoDirectoryStream();

    //! close an image stream
    void close();

    //! read image from stream
    void read(Video<uint8> &, int = 0, int = INT_MAX) throw (MocaException);

private:
    //! list of strings containing files in directory
    NumberedFileList *nfl;      // (c)

    //! called by constructor and open
    void initialise(const string &) throw (MocaException);

    //! called by destructor and close
    void finalise();

}; // class IVideoStream

#endif


