/*
   This file is part of the MoCA video library.
   Name: IVideoFileStream.cpt
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: IVideoFileStream.cpt,v 1.2 2004/03/05 15:21:24 kopf Exp $

*/

#include <IVideoFileStream.h>


/*!
 */
inline
IVideoFileStream::IVideoFileStream(const string &uri)
{
    open(uri);
} // IVideoFileStream::IVideoFileStream


/*!
 */
void
IVideoFileStream::open(const string &uri) throw (MocaException)
{
    // open file stream
	is = NULL;
    is = new ifstream(uri.c_str(), std::ios::in | std::ios::binary);
	if (!is) {
        string temp("IVideoFileStream::open  Could not access file ");
        temp = temp + uri;
        throw MocaException(temp);
	} // if
	if (!(is->rdstate() == (std::ios::goodbit))) {
        string temp("IVideoFileStream::open  Could not open file ");
        temp = temp + uri;
        throw MocaException(temp);
	} // if

    // get format specific stream based on "is"
    getFormatStream();

} // IVideoFileStream::open


/*!
 */
inline
IVideoFileStream::~IVideoFileStream()
{
    close();
} // IVideoFileStream::~IVideoFileStream


/*!
 */
void
IVideoFileStream::close()
{
    if (is) {
        delete is;
    } // if

    is = NULL;

    if (ivffs) {
        delete ivffs;
    } // if

    ivffs = NULL;
} // IVideoFileStream::close

/*!
 */
inline
void
IVideoFileStream::read(Video<uint8> &dest, int start, int num) throw (MocaException)
{
    if (ivffs == NULL) {
        throw MocaException("Trying to read from closed istream.");
    } // if

    ivffs->read(dest, start, num);
} // IVideoFileStream::read


/*!

  search for an appropriate format stream in arbitrary order

 */
void
IVideoFileStream::getFormatStream()
{
    ivffs = NULL;
    ivffs = new IPgmVideoFormatStream(*is);

    if (ivffs->checkFormat()) {
        return;
    } // if
    delete ivffs;
	ivffs = NULL;

	ivffs = NULL;
    ivffs = new IPpmVideoFormatStream(*is);
    if (ivffs->checkFormat()) {
        return;
    } // if
    delete ivffs;
	ivffs = NULL;


	//ivffs = NULL;
    //ivffs = new IMpegVideoFormatStream(*is);

    throw MocaException("Unknown Video File Format.");
} // IVideoFileStream::getFormatStream

