/*
   This file is part of the MoCA video library.
   Name: IVideoFileStream.h
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file


   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: IVideoFileStream.h,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

#ifndef I_VIDEO_FILE_STREAM_H
#define I_VIDEO_FILE_STREAM_H

// C++ includes
#include <string>
#include <fstream>
using namespace std;

// MoCA includes
#include <MocaExceptions.h>
#include <Video.h>
#include <IVideoStream.h>
#include <IVideoFileFormatStream.h>

// MoCA includes, format specific stream types
#include <IPgmVideoFormatStream.h>
#include <IPpmVideoFormatStream.h>


/*!

  Input stream for image files

 */
class IVideoFileStream : public IVideoStream
{
public:
    //! the default constructor
    IVideoFileStream() : is(NULL), ivffs(NULL) {};

    //! construct stream based on url provided
    IVideoFileStream(const string &);

    //! open an image stream
    void open(const string &) throw (MocaException);

    //! the destructor
    ~IVideoFileStream();

    //! close an image stream
    void close();

    //! read image from stream
    void read(Video<uint8> &, int = 0, int = INT_MAX) throw (MocaException);

private:
    //! raw stream
    istream *is;

    //! format specific stream based on is
    IVideoFileFormatStream *ivffs;

    //! build "ivffs" based on "is"
    void getFormatStream();

}; // class IVideoFileStream

#endif
