/*
   This file is part of the MoCA video library.
   Name: IVideoStream.cpt
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: IVideoStream.cpt,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

// this needs to be here to prevent circular include problems
#include <IVideoStream.h>
#include <IVideoDirectoryStream.h>
#include <IVideoFileStream.h>

/*!
 */
inline
IVideoStream::IVideoStream(const string &uri)
        : ivs(NULL)
{
    initialise(uri);
} // IVideoStream::IVideoStream


/*!
 */
inline
void IVideoStream::open(const string &uri)
{
    initialise(uri);
} // IVideoStream::open


/*!

  do the work of constructor and open

 */
inline
void IVideoStream::initialise(const string &uri) throw (MocaException)
{
    if (ivs != NULL) {
        throw MocaException("Close stream before opening it...");
    } // if

    // check for asterix and decide between directory and file
    if (uri.find('*') != string::npos) {
//cerr << "IVideoStream.cpt, line 89: DEBUG" << endl;
//        ivs = (IVideoStream*) new IVideoDirectoryStream<T>(uri);
    } else {
//cerr << "IVideoStream.cpt, line 92: DEBUG" << endl;
//        ivs = (IVideoStream*) new IVideoFileStream<T>(uri);
    } // if

} // IVideoStream::initialise


/*!
 */
inline 
IVideoStream::~IVideoStream()
{
    finalise();
} // IVideoStream::~IVideoStream


/*!
 */
inline
void IVideoStream::close()
{
    finalise();
} // IVideoStream::close


/*!

  do the work of destructor and close

 */
void IVideoStream::finalise()
{
    if (ivs != NULL) {
        delete ivs;
    } // if

    ivs = NULL;
} // IVideoStream::finalise


/*!
 */
inline
void
IVideoStream::read(Video<uint8> &dest, int start, int num) throw (MocaException)
{
    if (ivs == NULL) {
        throw MocaException("Trying to read from closed istream.");
    } // if

    ivs->read(dest, start, num);
} // IVideoStream::read

