/*
   This file is part of the MoCA video library.
   Name: MocaIo.h
   Copyright (C) 2006 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef MOCA_IO_H
#define MOCA_IO_H

//
// just include this file, if you have to include many different io routines (headers) in your file
//
// include all header files in directory "io"
#include <AudioDecoder.h>
#include <CreateMocaPath.h>
#include <FileHandling.h>
#include <FileList.h>
#include <FileReader.h>
#include <FormatedText.h>
#include <FormatParameter.h>
#include <IImageFormatStream.h>
#include <IImageStream.h>
#include <IJpegFormatStream.h>
#include <IPgmFormatStream.h>
#include <IPgmVideoFormatStream.h>
#include <IPngFormatStream.h>
#include <IPpmFormatStream.h>
#include <IPpmVideoFormatStream.h>
#include <ITagStream.h>
#include <IVideoDirectoryStream.h>
#include <IVideoFileFormatStream.h>
#include <IVideoFileStream.h>
#include <IVideoStream.h>
#include <IWaveAudio.h>
#include <IYuvVideo.h>
#include <JpegParameter.h>
#include <MpegStatus.h>
#include <NumberedFileList.h>
#include <OImageFormatStream.h>
#include <OImageStream.h>
#include <OJpegFormatStream.h>
#include <OMpegAudio.h>
#include <OMpegMplex.h>
#include <OMpegVideo.h>
#include <OpenFile.h>
#include <OpenProcess.h>
#include <OPgmFormatStream.h>
#include <OPgmVideoFormatStream.h>
#include <OPngFormatStream.h>
#include <OPpmFormatStream.h>
#include <OPpmVideoFormatStream.h>
#include <OTagStream.h>
#include <OVideoFormatStream.h>
#include <OWaveAudio.h>
#include <OYuvVideo.h>
#include <PgmParameter.h>
#include <PngParameter.h>
#include <PpmParameter.h>
#include <PpmVideoDecoder.h>
#include <PpmVideoEncoder.h>
#include <ProcessReader.h>
#include <ReadMocaData.h>
#include <TagFormat.h>
#include <VideoDecoder.h>
#include <VideoDecoderBase.h>
#include <VideoEncoder.h>
#include <VideoEncoderBase.h>
#include <VideoIndex.h>
#include <WaveHeader.h>
#include <WriteMocaData.h>
#include <XmlFormat.h>

void testIo ()
{
	// define path with test images
	string libMocaPath="../";  // linux path
#ifdef WIN32
	libMocaPath="../../../../libmoca/";  // windows path
#endif

	// global variables
	string fn="";


	// open JPG image
	fn=libMocaPath + "doc/lena.jpg";
	cerr << "read image: " << fn << endl;
	Image<uint8> img;
	ReadImage (fn, img);

	// open PNG image
	fn=libMocaPath + "doc/lena.png";
	cerr << "read image: " << fn << endl;
	ReadImage (fn, img);

	// open PGM image
	fn=libMocaPath + "doc/lena.pgm";
	cerr << "read image: " << fn << endl;
	ReadImage (fn, img);

	// open PPM image
	fn=libMocaPath + "doc/lena.ppm";
	cerr << "read image: " << fn << endl;
	ReadImage (fn, img);



#if 0
	// check/initialize path of avi file
	fn=libMocaPath + "doc/test.avi";
	CreateMocaPath cmp (fn, false);
	string aviFn = cmp.pathFile;
	cerr << "check avi file: " << aviFn << endl;
#endif

#if 0
#include <AudioDecoder.h>
#include <FileHandling.h>
#include <FileList.h>
#include <FileReader.h>
#include <FormatedText.h>
#include <FormatParameter.h>
#include <IPgmVideoFormatStream.h>
#include <IPpmVideoFormatStream.h>
#include <ITagStream.h>
#include <IVideoDirectoryStream.h>
#include <IVideoFileFormatStream.h>
#include <IVideoFileStream.h>
#include <IVideoStream.h>
#include <IWaveAudio.h>
#include <IYuvVideo.h>
#include <JpegParameter.h>
#include <MpegStatus.h>
#include <NumberedFileList.h>
#include <OImageFormatStream.h>
#include <OImageStream.h>
#include <OJpegFormatStream.h>
#include <OMpegAudio.h>
#include <OMpegMplex.h>
#include <OMpegVideo.h>
#include <OpenFile.h>
#include <OpenProcess.h>
#include <OPgmFormatStream.h>
#include <OPgmVideoFormatStream.h>
#include <OPngFormatStream.h>
#include <OPpmFormatStream.h>
#include <OPpmVideoFormatStream.h>
#include <OTagStream.h>
#include <OVideoFormatStream.h>
#include <OWaveAudio.h>
#include <OYuvVideo.h>
#include <PgmParameter.h>
#include <PngParameter.h>
#include <PpmParameter.h>
#include <PpmVideoDecoder.h>
#include <PpmVideoEncoder.h>
#include <ProcessReader.h>
#include <ReadMocaData.h>
#include <TagFormat.h>
#include <VideoDecoder.h>
#include <VideoDecoderBase.h>
#include <VideoEncoder.h>
#include <VideoEncoderBase.h>
#include <VideoIndex.h>
#include <WaveHeader.h>
#include <WriteMocaData.h>
#include <XmlFormat.h>
#endif
	return;
}

#endif

