/*
    OMpegAudio.cpp     store audio files
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <OMpegAudio.h>

const string OMpegAudio::WINDOWS_TOOLAME_EXECUTABLE = "toolame.exe";
const string OMpegAudio::LINUX_TOOLAME_EXECUTABLE   = "toolame";


// constuctor setting filename
OMpegAudio::OMpegAudio (const string &destFilename, int samplesPerSec, int channels)
{	
#ifdef COMPILER_MSVC
	// Switch Win32 standard output to binary mode, so that piping the
	// raw image data works. 
	{
		int result = _setmode( _fileno(stdout), _O_BINARY );
		if (result == -1) {
			cerr << "Cannot set standard output to binary mode.\n";
			exit(1);
		} // if
	}
	// Switch Win32 standard input to binary mode, so that piping the
	// raw image data works. 
	{
		int result = _setmode( _fileno(stdin), _O_BINARY );
		if (result == -1) {
			cerr << "Cannot set standard input to binary mode.\n";
			exit(1);
		} // if
	}
#endif
	
	fh_out = NULL;


	// modify filename, if spaces in filename/path exist
	outputMp2Filename = destFilename;
	CreateMocaPath pathCheck (outputMp2Filename, false);
	pathCheck.validatePath (outputMp2Filename);

	
	this->samplesPerSec = "44.1";
	switch (samplesPerSec) {
	case 16000:
		this->samplesPerSec = "16.0";
		break;
	case 22050:
		this->samplesPerSec = "22.05";
		break;
	case 24000:
		this->samplesPerSec = "24.0";
		break;
	case 32000:
		this->samplesPerSec = "32.0";
		break;
	case 44100:
		this->samplesPerSec = "44.1";
		break;
	case 48000:
		this->samplesPerSec = "48.0";
		break;
	default:
		cerr << endl 
		     << "OMpegAudio: " << samplesPerSec << " samples per second not supported."<< endl;
		cerr << "            using default samples per second (44100)!" << endl << endl;
	} // switch

	this->channels = channels;
	
	initEncoder();

} // OMpegAudio

// static function: convert WAV-input file to mp2-file
void
OMpegAudio::Wav2Mp2 (const string &srcFilenameWav, const string &destFilenameMp2, int samplesPerSecond, int channels)
{	
    string tmpSrcWav;
    string tmpDestMp2;
    FileHandling::convertToValidUnixFilename(srcFilenameWav,  tmpSrcWav);
    FileHandling::convertToValidUnixFilename(destFilenameMp2, tmpDestMp2);
	
    string sampPerSec= "44.1";
    switch (samplesPerSecond) {
	case 16000:
	    sampPerSec = "16.0";
	    break;
	case 22050:
	    sampPerSec = "22.05";
	    break;
	case 24000:
	    sampPerSec = "24.0";
	    break;
	case 32000:
	    sampPerSec = "32.0";
	    break;
	case 44100:
	    sampPerSec = "44.1";
	    break;
	case 48000:
	    sampPerSec = "48.0";
	    break;
	default:
	    cerr << endl 
		 << "OMpegAudio: " << samplesPerSecond << " samples per second not supported."<< endl;
	    cerr << "            using default samples per second (44100)!" << endl << endl;
    } // switch


    // define command
    string cmd = getMp2Encoder();

    // set channels
    string ch = "";
    if (channels == 1) {
	ch = " -m m ";
    } // if

    // set command: "toolame -s 44.1  WAV-sourcefile  MP2-destfile    
    cmd = cmd + " -s " + sampPerSec + ch + "  " + tmpSrcWav + "  " + tmpDestMp2;
    cerr << "cmd: " << cmd << endl;

    // open process
    system(cmd.c_str());

} // OMpegAudio



// destructor
OMpegAudio::~OMpegAudio ()
{	
	close();

} // ~OMpegAudio


	// initialize audio encoder 
void
OMpegAudio::initEncoder () 
{
	if (fh_out != NULL) {
		pclose (fh_out);
	} // if
	fh_out = NULL;


	// define command
	string cmd = getMp2Encoder();

	string ch = "";
	if (channels == 1) {
		ch = " -m m ";
	} // if

	// set command
	cmd = cmd + " -s " + samplesPerSec + ch + " - " + outputMp2Filename;

	// open process
	OpenProcess::open(&fh_out, cmd, P_WRITE);
	return;
} // audioDecoder


// close
void
OMpegAudio::close() 
{
	// close file handler
	if (fh_out) {
		pclose(fh_out);
	} // if
	fh_out = NULL;

	return;
} // close



// write raw data
void
OMpegAudio::write(Signal<int16>& sig) 
{

	SignalSize size = sig.getSize();
	if (size[DIM_X]<=0)  {
		return;
	} // if
	int16 *sigPtr0 = sig.getSamplePtr(0,0);
	int16 *sigPtr1 = sig.getSamplePtr(0,1);
	
	int32 v;
	int32 counter;
	uint8 buf[4];
	if (channels == 1) {
		// write to stdout
		for (int32 i=0; i<size[DIM_X]; i++) {
			v = *sigPtr0++;
			buf[0] = v;
			buf[1] = (v >> 8);
			
			counter = 0;		
			counter = fwrite(buf,1, 2, fh_out);
			if (counter != 2) {
				cerr << "OMpegAudio: could not write raw data." << endl << endl;
				exit (1);						
			} // if
		} // for
	} // if

	if (channels == 2) {
		// save data
		for (int32 i=0; i<size[DIM_X]; i++) {
			v = *sigPtr0++;
			buf[0] = v;
			buf[1] = (v >> 8);
			v = *sigPtr1++;
			buf[2] = v;
			buf[3] = (v >> 8);

			counter = 0;		
			counter = fwrite(buf,1, 4, fh_out);
			if (counter != 4) {
				cerr << "OMpegAudio: could not write raw data." << endl << endl;
				exit (1);						
			} // if

		} // for
	} // if

	// flush buffer
	counter = fflush(fh_out);

	return;
} // close

