/*
   This file is part of the MoCA video library.
   Name: OMpegVideo.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

 OMpegVideo.h -- Interface for MPEG1/MPEG2 video
 
  Author: Stephan Kopf
  Lehrstuhl fuer Praktische Informatik IV
  Universitaet Mannheim, Germany
  email: kopf@pi4.informatik.uni-mannheim.de
  
   
*/

#ifndef O_MPEG_VIDEO_H
#define O_MPEG_VIDEO_H


// MoCA includes

#include <MocaTypes.h>
#include <Image.h>
#include <Video.h>
#include <OpenProcess.h>
#include <OpenFile.h>



#ifdef COMPILER_MSVC

#include <process.h>

#include <fcntl.h>

#include <io.h>

#endif




// C includes
#include <stdio.h>


// C++ includes
#include <sstream>
#include <string>
#include <iostream>
using namespace std;





const string WINDOWS_SAMPEG_EXECUTABLE = "sampeg.exe";
const string LINUX_SAMPEG_EXECUTABLE   = "sampeg";

class OMpegVideo 
{
	
public:
	
	// constuctor setting filename
	OMpegVideo (const string &pathFilename, ImageSize &size, ieee64 framerate, ieee64 bitrate, bool createGrayscale, bool cropTo16);
	
	// destructor
	~OMpegVideo ();
	
	// write frame
	void write(const Image<uint8> &src);
	void close();
	
private:
	void open();
	void writeData(const unsigned char *buf, int size);
	inline int Clip(int x) 
	{ 
		if (x<0) return 0; 
		if (x>255) return 255; 
		return x; 
	} // Clip

	
	FILE *fh_out;
	int32 fullResolution;
	int32 quarterResolution;
	int32 bufSize; 	      // full resolution (Y), 2 * quarter resolution (U,V)
	unsigned char* buffer;


	string cmd;
	string pathFilename;
	ImageSize size;	
	int error;
	ieee64 framerate;
	ieee64 bitrate;
	bool createGrayscale;


	bool cropTo16;
	ImageSize cropedImageSize;
	int cropLeft;
	int cropRight;
	int cropTop;
	int cropBottom;

};

class WriteVideo
{
public:
	WriteVideo (Video<uint8> &vid, const string &filename)
	{
		process (filename, vid);
	}


private:
	void process (const string &filename, Video<uint8> &vid)
	{
		try {
			// open writer
			VideoSize size = vid.getSize();
			ImageSize imgSize (size[DIM_X], size[DIM_Y], size[DIM_BAND]);
			ieee64 framerate = 25.0;
			ieee64 bitrate = 1500000;
			bool createGrayscale=false;
			bool cropTo16 = true;
			OMpegVideo writer (filename, imgSize, framerate, bitrate, createGrayscale, cropTo16);
			for (int32 i=0; i<size[DIM_Z]; i++) {
				Image<uint8> img = vid.getImage(i);
				writer.write(img);	
			} // for
			writer.close();

		} catch (MocaException e) {
			cout << "WriteVideo: " << e.what() << endl;
			exit (-1);
		} // try
	} // process

};


#endif
