/* -*- C++ -*-

   OpenProcess.cpp  -- opens an external executable  

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <OpenProcess.h>

  
void 
OpenProcess::open(FILE **d_fh, string &cmd, OpenProcessMode mode, bool silent) 
{

	// set flags
	string processOpenFlags = "";
	switch (mode) {
	    case P_READ:
		processOpenFlags = "r";
#ifdef _WIN32
		processOpenFlags = "rb";
#endif
		break;
	    case P_WRITE:
		processOpenFlags = "w";
#ifdef _WIN32
		processOpenFlags = "wb";
#endif
		break;
	    default:
		cerr << "OpenProcess::unknown mode, please use P_READ or P_WRITE." << endl;
		cerr << "             cannot open process: " << cmd << endl;
		exit(1);
		break;
	} // switch

	// print
	if (!silent) {
		cerr << "open process: (" << processOpenFlags << ")  (" << cmd << ") " << endl; 
	} // if
	// open process
	(*d_fh)=NULL;
	(*d_fh) = popen(cmd.c_str(),processOpenFlags.c_str());

	// is error?
	if ((*d_fh) == NULL) {
	    cerr << "OpenProcess::cannot open process: " << cmd << "    (fh==NULL)" << endl;
	    exit(1);
	} // if

	// is error?
	int code = ferror((*d_fh));
	if (code != 0) {
	    cerr << "OpenProcess::cannot open process: " << cmd << "    (ferror)" << endl;
	    exit(1);
	} // if

	// is error?
	code = feof((*d_fh));
	if (code != 0) {
	    cerr << "OpenProcess::cannot open process: " << cmd << "    (ferror)" << endl;
	    exit(1);
	} // if
	
	return;
} // open

