/*
    PpmVideoEncoder.cpp     abstract class to encode videos
    Copyright (C) 2005 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <PpmVideoEncoder.h>


PpmVideoEncoder::PpmVideoEncoder (const string &pathFile) : VideoEncoderBase (pathFile)
{
  buf=NULL;
  binary = true;
  init();
}


PpmVideoEncoder::~PpmVideoEncoder ()
{
  if (buf!=NULL) {
    delete[] buf;
  } // if
  buf=NULL;
}


// write next image 
void 
PpmVideoEncoder::write (Image<uint8> &img)
{
    // test for correct dimensions
    ImageSize size = img.getSize();
    if (size[DIM_BAND] != 3) {
      size[DIM_BAND]=3;
      Image<uint8> tmp (size);
      tmp.copy(img, SampleBlockPosition (0,0,0,0));
      tmp.copy(img, SampleBlockPosition (0,0,0,1));
      tmp.copy(img, SampleBlockPosition (0,0,0,2));
      storeFrame (tmp);	
      return;
    } // if

    storeFrame (img);	
    return;
}


// write next image 
void 
PpmVideoEncoder::storeFrame (Image<uint8> &img)
{
  bool error = false;
  try {
    ImageSize size = img.getSize();
    if (binary) {
      if (buf==NULL) {
	bufSize = size[DIM_X]*size[DIM_Y]*3;
	buf = new char[bufSize];
      } // if
      
      // store image in binary format
      *os << "P6" << endl
	  << size[DIM_X] << " " << size[DIM_Y] << endl
	  << "255" << endl;
      
      char* bufP = buf;
      for (int y = 0; y < size[DIM_Y]; y++) {
	for (int x=0; x<size[DIM_X]; x++) {
	  *bufP = img(x,y,0);
	  bufP++;
	  *bufP = img(x,y,1);
	  bufP++;
	  *bufP = img(x,y,2);
	  bufP++;
	} // for
      } // for
      
      os->write (buf, bufSize);
      os->flush();
    } else {
      
      // store image in raw format
      *os << "P3" << endl
	  << size[DIM_X] << " " << size[DIM_Y] << endl
	  << "255" << endl;
      
      for (int y = 0; y < size[DIM_Y]; y++) {
	for (int x=0; x<size[DIM_X]; x++) {
	  *os << (int32) img(x,y,0) << " ";
	  *os << (int32) img(x,y,1) << " ";
	  *os << (int32) img(x,y,2) << endl;
	} // for
      } // for
    } // else
  } catch(...) {
    error = true;
  } // catch
  if (os->fail()) {
    error = true;
  } // if
  if (error) {
    cerr << "Error in PpmVideoEncoder, storeFrame: cannot store frame to PPM file!" << endl << endl;
    exit(1);
  } // if
  
  return;
}


// write video
void
PpmVideoEncoder::write (Video<uint8> &vid)
{
  int32 size = vid.getSize()[DIM_Z];
  Image<uint8> tmp;
  for (int32 i=0; i<size; i++) {
    cerr << "\r" << i << "  " << ends;
    tmp = vid.getImage(i);
    write(tmp);
  } // for
  cerr << endl;
  return;
}


// write video
void
PpmVideoEncoder::write (ImageVector<uint8> &vid)
{
  int32 size = vid.getSize();
  for (int32 i=0; i<size; i++) {
    cerr << "\r" << i << "  " << ends;
    write(vid.data[i]);
  } // for
  cerr << endl;
  return;
}



// init
void
PpmVideoEncoder::init ()
{
  // open input stream
  open();
  
  return;
}

