/*
   This file is part of the MoCA video library.
   Name: PpmVideoEncoder.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

  PpmVideoEncoder.h -- encodes PPM video sequences
                     
 
  Author: Stephan Kopf
  Lehrstuhl fuer Praktische Informatik IV
  Universitaet Mannheim, Germany
  email: kopf@pi4.informatik.uni-mannheim.de
  

*/

#ifndef PPM_VIDEO_ENCODER_H
#define PPM_VIDEO_ENCODER_H


// MoCA includes
#include <VideoEncoderBase.h>


// this class provides the basic routines to encode a video
class PpmVideoEncoder : VideoEncoderBase
{
public:
  
  // constuctor defines filename 
  PpmVideoEncoder (const string &pathFile);
  
  // destructor
  ~PpmVideoEncoder ();
  
  // set additional parameters
  void setParameters (bool storeBinary)
  {
    binary = storeBinary;
    return;
  }
  
  // store frames
  void write (Image<uint8> &img);
  void write (Video<uint8> &vid);
  void write (ImageVector<uint8> &vid);
  
 private:
  void init ();
  void storeFrame (Image<uint8> &img);
  
  bool binary;
  int32 bufSize;
  char* buf;
};


#endif
