/*
   This file is part of the MoCA video library.
   Name: SampleBlockSize.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   file.h  -- short description

   Author:    Gerald Kuehne
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kuehne@pi4.informatik.uni-mannheim.de

   $Id: SampleBlockSize.h,v 1.1 2004/02/17 12:00:58 kopf Exp $
*/

#ifndef SAMPLE_BLOCK_SIZE_H
#define SAMPLE_BLOCK_SIZE_H

// MoCA includes
#include <MocaTypes.h>
#include <MocaEnum.h>
#include <MocaMacros.h>

class SampleBlockSize
{
public:
  SampleBlockSize ();
  SampleBlockSize (const SampleBlockSize&);
  SampleBlockSize (sizeType x, sizeType y, sizeType z, sizeType bands);

  sizeType& operator[](sizeType i);
  const sizeType& operator[](sizeType i) const;

  int operator== (const SampleBlockSize&) const;
  int operator!= (const SampleBlockSize&) const;
  SampleBlockSize& operator= (const SampleBlockSize&);


private:
  sizeType size[DIM_N];
};

inline
SampleBlockSize::
SampleBlockSize ()
{
  size[DIM_X]    = 0;
  size[DIM_Y]    = 0;
  size[DIM_Z]    = 0;
  size[DIM_BAND] = 0;
}

inline
SampleBlockSize::
SampleBlockSize (sizeType x, sizeType y, sizeType z, sizeType bands)
{
  size[DIM_X]    = x;
  size[DIM_Y]    = y;
  size[DIM_Z]    = z;
  size[DIM_BAND] = bands;
}

inline
SampleBlockSize::
SampleBlockSize (const SampleBlockSize &other)
{
  size[DIM_X]    = other.size[DIM_X];
  size[DIM_Y]    = other.size[DIM_Y];
  size[DIM_Z]    = other.size[DIM_Z];
  size[DIM_BAND] = other.size[DIM_BAND];
}

inline
sizeType&
SampleBlockSize::
operator[] (sizeType i)
{
    return size[i];
}

inline
const sizeType&
SampleBlockSize::
operator[] (sizeType i) const
{
  return size[i];
}

inline
int
SampleBlockSize::operator== (const SampleBlockSize& other) const
{
  return(size[DIM_X] == other.size[DIM_X] &&
         size[DIM_Y] == other.size[DIM_Y] &&
         size[DIM_Z] == other.size[DIM_Z] &&
         size[DIM_BAND] == other.size[DIM_BAND]);
}

inline
int
SampleBlockSize::operator!= (const SampleBlockSize& other) const
{
  return !(*this == other);
}

inline
SampleBlockSize&
SampleBlockSize::operator= (const SampleBlockSize& other) 
{
	if (this != &other) {
		size[DIM_X]    = other.size[DIM_X];   
		size[DIM_Y]    = other.size[DIM_Y];   
		size[DIM_Z]    = other.size[DIM_Z];
		size[DIM_BAND] = other.size[DIM_BAND];
	} // if
	return *this;
} // operator=


#endif





